<?php class dropimage extends baseobject {
	
	protected static $fields=array(
			'name'=>validator::REQUIRED,
			'imagingsessionid'=>array(validator::REQUIRED, validator::INTEGER),
			'welldropid'=>array(validator::REQUIRED, validator::INTEGER),
			'pixelheight'=>array(validator::REQUIRED, validator::INTEGER),
			'pixelwidth'=>array(validator::REQUIRED, validator::INTEGER),
			'micronsperpixelx'=>array(validator::REQUIRED,validator::FLOAT),
			'micronsperpixely'=>array(validator::REQUIRED,validator::FLOAT),
			'latestcrystalscoreid'=>validator::INTEGER,
			'imagestorepath'=>validator::REQUIRED,
			'imagepath'=>validator::REQUIRED,
			'thumbnailpath'=>validator::REQUIRED,
	);
	
	protected static $helpTexts=array(
			'name'=>'A unique name for this drop image',
			'imagingsessionid'=>'The imaging session where this image was taken',
			'welldropid'=>'The drop that was imaged',
			'pixelheight'=>'Height of the image in pixels',
			'pixelwidth'=>'Width of the image in pixels',
			'micronsperpixelx'=>'Scale on the x axis',
			'micronsperpixely'=>'Scale on the y axis',
			'imagestorepath'=>'The base directory of the image store',
			'imagepath'=>'Path from image store to the main image',
			'thumbnailpath'=>'Path from image store to thumbnail',
	);
	
	public static $defaultSortOrder='platewell.row, platewell.col, welldrop.dropnumber, imagingsession.imageddatetime';
	
	protected static $adminSelect='SELECT SQL_CALC_FOUND_ROWS dropimage.*, imagingsession.imageddatetime, imagingsession.lighttype,
			CONCAT("/api/dropimagefile/",dropimage.id,"/full.",SUBSTRING_INDEX(dropimage.imagepath,".",-1) ) AS fullimageurl, 
			CONCAT("/api/dropimagethumb/",dropimage.id,"/thumb.",SUBSTRING_INDEX(dropimage.thumbnailpath,".",-1)) AS thumbnailurl,
			platewell.id AS platewellid, platewell.row, platewell.col, welldrop.dropnumber
			FROM dropimage, welldrop, platewell, imagingsession 
			WHERE dropimage.welldropid=welldrop.id AND welldrop.platewellid=platewell.id AND dropimage.imagingsessionid=imagingsession.id ';
	protected static $normalSelect='SELECT SQL_CALC_FOUND_ROWS dropimage.*, imagingsession.imageddatetime, imagingsession.lighttype, 
			CONCAT("/api/dropimagefile/",dropimage.id,"/full.",SUBSTRING_INDEX(dropimage.imagepath,".",-1) ) AS fullimageurl, 
			CONCAT("/api/dropimagethumb/",dropimage.id,"/thumb.",SUBSTRING_INDEX(dropimage.thumbnailpath,".",-1)) AS thumbnailurl,
			platewell.id AS platewellid, platewell.row, platewell.col, welldrop.dropnumber
			FROM dropimage, welldrop, platewell, imagingsession 
			WHERE dropimage.welldropid=welldrop.id AND welldrop.platewellid=platewell.id AND dropimage.imagingsessionid=imagingsession.id ';
	
	public static function getSelectClause(){
		return static::$normalSelect;
	}
	
	protected static function getProjectClause($accessType){
		return ''; //Images are publicly accessible, nothing in the metadata worth hiding.
	}
	
	
	public static function update($id,$request=array()){
		try {
			//set the latest score onto this image - throws exception if no permissions
			$ret=parent::update($id, $request);
			$now=date("Y-m-d H:i:s");
			if(isset($request['latestcrystalscoreid'])){
				//record the score in the dedicated table
				$crystalScore=crystalscore::getById($request['latestcrystalscoreid']);
				$crystalScoreIndex=(int)($crystalScore['scoreindex']);
				$sqlStatement='INSERT INTO scoreofdropimage(name,projectid,dropimageid,crystalscoreid,scoreddatetime) 
						VALUES(:name,:projectid,:imageid,:scoreid,:time)';
				$parameters=array(
						':name'=>'img'.$id.' score'.$request['latestcrystalscoreid'].' at '.$now,
						':projectid'=>$ret['updated']['projectid'],
						':imageid'=>$id,
						':scoreid'=>$request['latestcrystalscoreid'],
						':time'=>$now
				);
				database::query($sqlStatement, $parameters);
				
				$dropImage=dropimage::getById($id);
				$plateWell=platewell::getById($dropImage['platewellid']);
				$plate=plate::getById($plateWell['plateid']);
				$plateBestScoreIndex=-1;
				if(!empty($plate['bestcrystalscoreid'])){
					$bestScore=crystalscore::getById($plate['bestcrystalscoreid']);
					$plateBestScoreIndex=(int)($bestScore['scoreindex']);
				}
				
				if($plateBestScoreIndex<$crystalScoreIndex){
					$sqlStatement='UPDATE plate set bestcrystalscoreid=:bestid WHERE id=:plateid';
					$parameters=array(
							':bestid'=>$request['latestcrystalscoreid'],
							':plateid'=>$plate['id']
					);
					database::query($sqlStatement, $parameters);
				}
			}
			return $ret;
		} catch(Exception $e){
			throw $e;
		}
	}
	
}

?>