/**
 * This file contains functions specific to the drop viewer ( /imagingsession/NNN ).
 */

/**********************************************************************************************************
 * KEYBOARD SHORTCUTS HANDLER
 * Note that the actual key mappings are defined in dropviewer_keymappings.js
 **********************************************************************************************************/

var KeyboardShortcuts={

		ctrlPressed:false,
		detectCtrlKey:function(evt){
			if((evt.keyCode && evt.keyCode==17) || (evt.key && "Control"==evt.key)){
				if(evt.type=="keydown"){
					KeyboardShortcuts.ctrlPressed=true;
				} else if(evt.type=="keyup"){
					KeyboardShortcuts.ctrlPressed=false;
				}
			}
		},
		
		keyMapping:new Map([]),
		
		init:function(){
			var mapping=Cookies.get("dropviewer.keyboardMapping");
			if(!mapping){
				Cookies.set("dropviewer.keyboardMapping", "British");
				mapping="British";
			}
			KeyboardShortcuts.keyMapping=KeyboardShortcuts.keyMappings[mapping];
			document.observe("keydown", KeyboardShortcuts.handleKeyPress);
			document.observe("keydown", KeyboardShortcuts.detectCtrlKey);
			document.observe("keyup", KeyboardShortcuts.detectCtrlKey);
		},

		handleKeyPress:function(evt){
			if(document.activeElement){
				var type=document.activeElement.tagName.toLowerCase();
				if("input"==type||"textarea"==type||"select"==type){ return false;}
			}
			var keyPressed;
			var keyHandler;
			if(evt.key){
				keyPressed=evt.key;
			} else if(evt.charCode){ 
//				keyPressed=String.fromCharCode(evt.charCode).toLowerCase();
//				keyHandler=KeyboardShortcuts.keyMapping.get(keyPressed);
			} else if(evt.keyCode){
//				keyPressed=evt.keyCode;
//				if(evt.keyCode>=96 && evt.keyCode<=105){
//					//Numeric keypad. Subtract 48 to convert to regular number keys.
//					keyPressed=evt.keyCode-48;
//				}
//				keyHandler=KeyboardShortcuts.keyMapping.get(keyPressed);
//				if(!keyHandler){
//					keyPressed=String.fromCharCode(keyPressed).toLowerCase();
//					keyHandler=KeyboardShortcuts.keyMapping.get(keyPressed);
//				}
			}
			keyHandler=KeyboardShortcuts.keyMapping[keyPressed];
			if(keyHandler){
				keyHandler(keyPressed);
			}
		},

		movieFirst:function(){
			if(DropNav.isOpen){
				DropNav.goToFirst();
			} else if(currentPanel.goToFirst){
				currentPanel.goToFirst();
			} else {
				DropViewer.goToFirst();
			}
		},
		moviePrevious:function(){
			if(DropNav.isOpen){
				DropNav.goToPrevious();
			} else if(currentPanel.goToPrevious){
				currentPanel.goToPrevious();
			} else {
				DropViewer.goToPrevious(true);
			}
		},
		movieNext:function(){
			if(DropNav.isOpen){
				DropNav.goToNext();
			} else if(currentPanel.goToNext){
				currentPanel.goToNext();
			} else {
				DropViewer.goToNext(true);
			}
		},
		movieLast:function(){
			if(DropNav.isOpen){
				DropNav.goToLast();
			} else if(currentPanel.goToLast){
				currentPanel.goToLast();
			} else {
				DropViewer.goToLast();
			}
		},
		moviePlayStop:function(){
			if(DropNav.isOpen){
				return false;
			} else if(currentPanel.playOrStop){
				currentPanel.playOrStop();
			} else {
				DropViewer.playOrStop();
			}
		},

		upOneRow:function(){
			if(DropNav.isOpen){
				DropNav.goUpOneRow();
			} else if(currentPanel.goUpOneRow){
				currentPanel.goUpOneRow();
			} else {
				console.log("No goUpOneRow in "+currentPanel.name)
			}
		},
		downOneRow:function(){
			if(DropNav.isOpen){
				DropNav.goDownOneRow();
			} else if(currentPanel.goDownOneRow){
				currentPanel.goDownOneRow();
			} else {
				console.log("No goUpOneRow in "+currentPanel.name)
			}
		},
		
		toggleTimeCourse:function(){
			if(currentPanel==TimeCourse){
				switchPanels($("viewbutton_dv"));
			} else {
				switchPanels($("viewbutton_tc"));
			}
		}
		
}



/**********************************************************************************************************
 * DROP VIEWER
 **********************************************************************************************************/

var DropViewer={

	//array is zero-based, plate row numbers are 1-based, so empty item first.
	rowLabels:['','A','B','C','D','E','F','G','H','I','J','K','L','M','N','O','P','Q','R','S','T'],

	movieDelay:0.5, //seconds
	movieInterval:null,
	plateType:null,
	screen:null,
	currentIndex:0,
	maxIndex:0,
	images:[],
		
	init:function(){
		document.title="Drop viewer: Plate "+data['platename'];
		if(null!=Cookies.get("dropviewer.handedness")){
			setHandedness(Cookies.get("dropviewer.handedness"));
		}
		var mdelay=Cookies.get("dropviewer.movieDelay");
		if(mdelay){	DropViewer.movieDelay=mdelay; }
		$("dv_delay").innerHTML=DropViewer.movieDelay;
		$("tc_delay").innerHTML=DropViewer.movieDelay;
		DropViewer.getScoringSystem();
		DropViewer.getPlateType();
		DropViewer.getScreen();
		new Ajax.Request('/api/imagingsession/'+data['id']+'/dropimage?pagesize=10000',{
			method:'get',
			onSuccess:DropViewer.init_onSuccess,
			onFailure:DropViewer.init_onFailure
		});
		Event.observe(window, "resize", DropViewer.fitImageToPane);
	},

	init_onSuccess:function(transport){
		var images=transport.responseJSON.rows;
		var img=$("currentimage");
		img.src=images[0].fullimageurl;
		img.dataset.micronshigh=images[0].micronsperpixely*images[0].pixelheight;
		DropViewer.images=images;
		DropViewer.maxIndex=images.length-1;
		DropViewer.setCurrentImage(0);
		DropViewer.getCurrentImageTimeCourse();
		window.setTimeout(DropViewer.highlightCurrentImageScore,250);
		DropViewer.switchTo();
	},

	init_onFailure:function(transport){
		alert("Could not get images.");
	},

	getScreen:function(){
		if(""==data["screenid"] || null==data["screenid"] || 0==data["screenid"]){ return false; }
		new Ajax.Request('/api/screen/'+data["screenid"],{
			method:'get',
			onSuccess:DropViewer.getScreen_onSuccess,
			onFailure:DropViewer.getScreen_onFailure
		});
	},
	getScreen_onSuccess:function(transport){
		AjaxUtils.checkResponse(transport);
		window.setTimeout(function(){
			DropViewer.screen=transport.responseJSON;
			new Ajax.Request('/api/screen/'+data["screenid"]+'/screencondition?pagesize=10000&sortby=name',{
				method:'get',
				onSuccess:function(transport){ 
					AjaxUtils.checkResponse(transport);
					DropViewer.screen.conditions=transport.responseJSON.rows;
				},
				onFailure:function(transport){ /* do nothing, assume screen not set */ }
			});
		},50);
	},
	getScreen_onFailure:function(transport){
		/* do nothing, assume screen not set */
	},

	getPlateType:function(){
		new Ajax.Request('/api/platetype/'+data["platetypeid"],{
			method:'get',
			onSuccess:DropViewer.getPlateType_onSuccess,
			onFailure:DropViewer.getPlateType_onFailure
		});
	},
	getPlateType_onSuccess:function(transport){
		AjaxUtils.checkResponse(transport);
		window.setTimeout(function(){
			DropViewer.plateType=transport.responseJSON;
		},50);
	},
	getPlateType_onFailure:function(transport){
		alert("Could not retrieve plate type information");
	},
	
	
	getScoringSystem:function(){
		new Ajax.Request('/api/crystalscoringsystem/'+data["crystalscoringsystemid"]+'/score?sortby=scoreindex',{
			method:'get',
			onSuccess:DropViewer.getScoringSystem_onSuccess,
			onFailure:DropViewer.getScoringSystem_onFailure
		});
	},
	getScoringSystem_onSuccess:function(transport){
		AjaxUtils.checkResponse(transport);
		if(50>$("dv_scores").getHeight()){
			window.setTimeout(function(){DropViewer.getScoringSystem_onSuccess(transport)},50);
			return;
		}
		DropViewer.renderScoringSystem(transport);
	},
	
	/* If the score buttons would be below this height, render them double-height half-width.
	 * Intent: Make them more touchable on small tablets.
	 */
	scoreHeightCutoff:50,
	
	renderScoringSystem:function(transport){
		var scores=transport.responseJSON.rows;
		DropViewer.scores=scores;
		var total=transport.responseJSON.rows.length;
		var scoreHeight=Math.floor(  $("dv_scores").getHeight() / (1+Math.round(total)) );
		var renderHalfWidth=false;
		if(scoreHeight<DropViewer.scoreHeightCutoff){
			renderHalfWidth=true;
		}
		$("dv_scores").innerHTML="";
		scores.each(function(s){
			var sc=document.createElement("div");
			sc.id="score"+s.id;
			sc.dataset.scoreid=s.id;
			sc.dataset.scoreindex=s.scoreindex;
			sc.className="dv_score";
			sc.style.height=(scoreHeight-4)+"px";
			sc.style.backgroundColor="black";
			sc.style.borderColor="#"+s.color;
			sc.style.fontWeight="bold";
			sc.style.paddingLeft="1em";
			sc.style.textShadow="1px 1px 1px black";
			sc.innerHTML='['+s.scoreindex+']&nbsp; '+s.label;
			$("dv_scores").appendChild(sc);
			sc.style.borderColor="#"+s.color;
			if(renderHalfWidth){
				sc.style.lineHeight="auto";
				sc.style.height=(2*sc.getHeight()-10)+"px";
				sc.style.width="40%";
				sc.style.cssFloat="left";
			} else {
				sc.style.lineHeight=(sc.getHeight()-4)+"px";
				sc.style.height=(sc.getHeight()-4)+"px";
			}
			if(canUpdate){
				sc.style.cursor="pointer";
				sc.onclick=function(){ DropViewer.setScore(sc); }
			}
		});
		if(canUpdate){
			var lbl=document.createElement("label");
			var cb=document.createElement("input");
			cb.id="dv_advanceonscore";
			cb.type="checkbox";
			lbl.appendChild(cb);
			lbl.innerHTML+=" Go to next drop after scoring";
			lbl.style.borderRadius=0;
			lbl.style.clear="both"
			$("dv_scores").appendChild(lbl);
			lbl.style.lineHeight=lbl.previous().getHeight()+"px";
			lbl.style.textAlign="center";
			lbl.style.fontWeight="bold";
			window.setTimeout(function(){
				if(Cookies.get("dropviewer.advanceOnScore")==1){
					$("dv_advanceonscore").checked="checked";
				}
				$("dv_advanceonscore").observe("click",DropViewer.setAdvanceOnScore);
			},50);
		}
	},
	getScoringSystem_onFailure:function(transport){
		AjaxUtils.checkResponse(transport);
	},

	setAdvanceOnScore:function(){
		if($("dv_advanceonscore").checked){
			Cookies.set("dropviewer.advanceOnScore",1);
		} else {
			Cookies.set("dropviewer.advanceOnScore",0);
		}
	},

	setScoreByIndex:function(index){
		$$("#dv_scores .dv_score").each(function(s){
			if(s.dataset.scoreindex==index){
				DropViewer.setScore(s);
			}
		});
	},

	setScore:function(scoreElem){
		if(!canUpdate){ 
			alert("You don't have permission to update this plate. Talk to your administrator.");
			return false; 
		}
		//highlight it, unhighlight the rest
		$$("#dv_scores .dv_score").each(function(s){
			s.style.backgroundColor="black";
		});
		scoreElem.style.backgroundColor=scoreElem.style.borderColor;
		var scoreId=scoreElem.dataset.scoreid;
		var imageToScore=DropViewer.images[DropViewer.currentIndex];
		var imageId=imageToScore.id;
		imageToScore.latestcrystalscoreid=scoreId;
		new Ajax.Request('/api/dropimage/'+imageId, {
			method:'patch',
			parameters:{
				csrfToken:csrfToken,
				latestcrystalscoreid:scoreId
			},
			onSuccess:DropViewer.setScore_onSuccess,
			onFailure:DropViewer.setScore_onFailure,
		});
		
		if($("dv_advanceonscore").checked){
			window.setTimeout(function(){ DropViewer.goToNext(true)},200);
		}
	},
	setScore_onSuccess:function(transport){
		AjaxUtils.checkResponse(transport);
	},
	setScore_onFailure:function(transport){
		if(transport.responseJSON && transport.responseJSON.error  && transport.responseJSON.error.indexOf("Duplicate entry") >-1){ 
			//Scored the same drop twice with same score in same second - just eat it. 
			return false; 
		}
		AjaxUtils.checkResponse(transport);
	},
	
	increaseMovieDelay:function(){
		var md=DropViewer.movieDelay;
		md=(1*md+0.1).toFixed(1);
		Cookies.set("dropviewer.movieDelay",md);
		$("dv_delay").innerHTML=md;
		$("tc_delay").innerHTML=md;
		DropViewer.movieDelay=md;
		if(DropViewer.movieInterval){
			DropViewer.stopMovie();
			DropViewer.startMovie();
		}
	},
	decreaseMovieDelay:function(){
		var md=DropViewer.movieDelay;
		md=(1*md-0.1).toFixed(1);
		if(md<=0){ md=0.1; }
		Cookies.set("dropviewer.movieDelay",md);
		$("dv_delay").innerHTML=md;
		$("tc_delay").innerHTML=md;
		DropViewer.movieDelay=md;
		if(DropViewer.movieInterval){
			DropViewer.stopMovie();
			DropViewer.startMovie();
		}
	},
	
	switchTo:function(){
		if(DropViewer==currentPanel){ return false; }
		currentPanel=DropViewer;
		DropViewer.setCurrentImage(DropViewer.currentIndex);
		$$(".controlpanel").each(function(p){
			p.style.display="none";
		});
		$("dropviewerpanel").style.display="block";
		DropViewer.highlightCurrentImageScore();
		return true;
	},

	highlightCurrentImageScore:function(){
		var currentImageScore=DropViewer.images[DropViewer.currentIndex]['latestcrystalscoreid'];
		$$("#dv_scores .dv_score").each(function(s){
			if(s.dataset.scoreid==currentImageScore){
				s.style.backgroundColor=s.style.borderColor;
			} else {
				s.style.backgroundColor="black";
			}
		});
	},
	
	switchFrom:function(){
		DropViewer.stopMovie();
	},
	
	goToFirst:function(){
		DropViewer.stopMovie();
		DropViewer.setCurrentImage(0);
	},
	goToPrevious:function(byDrop){
		DropViewer.stopMovie();
		if(byDrop || KeyboardShortcuts.ctrlPressed){
			DropViewer.previousImage();
		} else {
			DropViewer.goLeftOneColumn();
		}
	},
	goToNext:function(byDrop){
		DropViewer.stopMovie();
		if(byDrop || KeyboardShortcuts.ctrlPressed){
			DropViewer.nextImage();
		} else {
			DropViewer.goRightOneColumn();
		}
	},
	goToLast:function(){
		DropViewer.stopMovie();
		DropViewer.setCurrentImage(DropViewer.images.length-1);
	},

	goLeftOneColumn:function(){
		var current=DropViewer.getCurrentImage();
		var newIndex=DropViewer.getImageIndexByRowColumnDrop(1*current.row,1*current.col-1,1*current.dropnumber);
		if(null!==newIndex){ 
			DropViewer.setCurrentImage(newIndex);
			return;
		}
		for(c=DropViewer.plateType.cols;c>=1;c--){
			newIndex=DropViewer.getImageIndexByRowColumnDrop(1*current.row-1,c,1*current.dropnumber);
			if(null!==newIndex){ 
				DropViewer.setCurrentImage(newIndex); 
				return;
			}
		}
		
	},
	goRightOneColumn:function(){
		var current=DropViewer.getCurrentImage();
		var newIndex=DropViewer.getImageIndexByRowColumnDrop(1*current.row,1*current.col+1,1*current.dropnumber);
		if(null!==newIndex){ 
			DropViewer.setCurrentImage(newIndex); 
			return;
		}
		for(c=1;c<=DropViewer.plateType.cols;c++){
			newIndex=DropViewer.getImageIndexByRowColumnDrop(1*current.row+1,c,1*current.dropnumber);
			if(null!==newIndex){ 
				DropViewer.setCurrentImage(newIndex); 
				return;
			}
		}
	},
	goUpOneRow:function(){
		var current=DropViewer.getCurrentImage();
		var newIndex=DropViewer.getImageIndexByRowColumnDrop(1*current.row-1,1*current.col,1*current.dropnumber);
		if(null!==newIndex){ DropViewer.setCurrentImage(newIndex); }
	},
	goDownOneRow:function(){
		var current=DropViewer.getCurrentImage();
		var newIndex=DropViewer.getImageIndexByRowColumnDrop(1*current.row+1,1*current.col,1*current.dropnumber);
		if(null!==newIndex){ DropViewer.setCurrentImage(newIndex); }
	},
	
	getImageIndexByRowColumnDrop:function(rowNum,colNum,dropNum){
		var numImages=DropViewer.images.length;
		for(i=0;i<numImages;i++){
			var img=DropViewer.images[i];
			if(img.row==rowNum && img.col==colNum && img.dropnumber==dropNum){
				return i;
			}
		}
		return null;
	},
	
	playOrStop:function(){
		if(DropViewer.movieInterval){
			DropViewer.stopMovie();
		} else {
			DropViewer.startMovie();
		}
	},
	startMovie:function(){
		if(DropViewer.currentIndex>=DropViewer.maxIndex){
			DropViewer.setCurrentImage(0);
		}
		DropViewer.movieInterval=window.setInterval(DropViewer.nextImage,DropViewer.movieDelay*1000);
		$("dv_playstop").title="Stop";
		$("dv_playstop").addClassName("playing");
		$("dv_timecourseends").style.display="none";
	},
	stopMovie:function(){
		window.clearInterval(DropViewer.movieInterval);
		DropViewer.movieInterval=null;
		$("dv_playstop").title="Play";
		$("dv_playstop").removeClassName("playing");
		DropViewer.getCurrentImageTimeCourse();
	},
	nextImage:function(){
		DropViewer.currentIndex++;
		if(!DropViewer.setCurrentImage(DropViewer.currentIndex)){
			DropViewer.currentIndex--;
			DropViewer.stopMovie();
		}
	},
	previousImage:function(){
		DropViewer.currentIndex--;
		if(!DropViewer.setCurrentImage(DropViewer.currentIndex)){
			DropViewer.currentIndex++;
			DropViewer.stopMovie();
		}
	},

	getCurrentImage:function(){
		return DropViewer.images[DropViewer.currentIndex];
	},
	
	setCurrentImage:function(index){
		index*=1;
		if(index>DropViewer.maxIndex){ return false; }
		if(index<0){ return false; }
		var img=DropViewer.images[index];
		$("currentimage").src=img.fullimageurl;
		$("currentimage").dataset.micronshigh=img.pixelheight*img.micronsperpixely;
		DropViewer.fitImageToPane(img);
		Measure.setScale();
		DropViewer.setDropNameInTitle(img);
		DropViewer.currentIndex=index;
		DropViewer.highlightCurrentImageScore();
		DropViewer.getCurrentImageTimeCourse();
		DropInfo.refresh();
		return true;
	},

	fitImageToPane:function(){
		var img=DropViewer.images[DropViewer.currentIndex];
		if(img.pixelwidth/$("imagepane").getWidth() > img.pixelheight/$("imagepane").getHeight()){
			$("currentimage").style.width="100%";
			$("currentimage").style.height="auto";
		} else {
			$("currentimage").style.width="auto";
			$("currentimage").style.height="100%";
		}
		Measure.setScale();
		if(CrystalSelection==currentPanel){
			CrystalSelection.redrawCrosshairs();
		}
	},
	
	getWellNameForImage:function(img){
		var col=img.col*1;
		if(col<10){ col="0"+col; }
		var wellName=DropViewer.rowLabels[img.row]+col;
		return wellName;
	},
	
	setDropNameInTitle:function(img){
		var dropName=DropViewer.getWellNameForImage(img)+"."+img.dropnumber;
		$("dropname").innerHTML=dropName;
		document.location.hash=dropName;
	},
	
	/**
	 * Retrieves all the image records for the current drop, as JSON. 
	 * Success handler attaches them to the current drop and updates first/last image.
	 */
	getCurrentImageTimeCourse:function(){
		//use a short timeout to break the call chain, otherwise pre-nav image's timecourse is fetched
		window.setTimeout(DropViewer._doGetCurrentImageTimeCourse,25);
	},
	_doGetCurrentImageTimeCourse:function(){
		var index=DropViewer.currentIndex;
		var obj=DropViewer.images[index];
		new Ajax.Request('/api/welldrop/'+obj.welldropid+'/timecourseimage?pagesize=1000',{
			method:'get',
			onSuccess:function(transport){ DropViewer.getCurrentImageTimeCourse_onSuccess(transport, DropViewer.currentIndex); },
			onFailure:function(transport){ DropViewer.getCurrentImageTimeCourse_onFailure(transport, DropViewer.currentIndex); },
		});
	},
	getCurrentImageTimeCourse_onSuccess:function(transport, imageIndex){
		var tc=transport.responseJSON.rows;
		DropViewer.images[imageIndex].timeCourseImages=tc;
		var first=$("dv_timecoursefirst");
		var last=$("dv_timecourselast");
		first.src=tc[0].thumbnailurl;
		first.dataset.fullsrc=tc[0].fullimageurl;
		first.onmouseover=function(){ DropViewer.swapInTimeCourseEndImage(first) };
		first.onmouseout=function(){ DropViewer.swapOutTimeCourseEndImage(first) };
		last.src=tc[tc.length-1].thumbnailurl;
		last.dataset.fullsrc=tc[tc.length-1].fullimageurl;
		last.onmouseover=function(){ DropViewer.swapInTimeCourseEndImage(last) };
		last.onmouseout=function(){ DropViewer.swapOutTimeCourseEndImage(last) };
		$("dv_timecourseends").style.display="block";
	},
	getCurrentImageTimeCourse_onFailure:function(transport, imageIndex){
		
	},

	swapInTimeCourseEndImage:function(img){
		img.dataset.oldsrc=$("currentimage").src;
		$("currentimage").src=img.dataset.fullsrc;
	},
	swapOutTimeCourseEndImage:function(img){
		$("currentimage").src=img.dataset.oldsrc;
		img.dataset.oldsrc='';
	},
	
}




/**********************************************************************************************************
 * TIME COURSE 
 **********************************************************************************************************/

var TimeCourse={

		//holds the src of the image showing before the time course was opened
		imageBefore:null,

		images:null,
		
		activeLightTypes:[],
		
		switchTo:function(){
			TimeCourse.imageBefore=$("currentimage").src.slice(0); //copy it
			if(TimeCourse==currentPanel){ return false; }
			$$(".controlpanel").each(function(p){
				p.style.display="none";
			});
			$("timecoursepanel").style.display="block";
			currentPanel=TimeCourse;
			var container=$("tc_images");
			container.innerHTML="Waiting for images...";
			TimeCourse.writeTimeCourseImages();
			return true;
		},
		switchFrom:function(){
			$("currentimage").src=TimeCourse.imageBefore;
			Measure.setScale();
		},

		writeTimeCourseImages:function(){
			var currentImage=DropViewer.getCurrentImage();
			while(!currentImage.timeCourseImages){
				//we wait for the DOM to catch up
			}
			var container=$("tc_images");
			container.innerHTML="";
			TimeCourse.images=currentImage.timeCourseImages;
			if(TimeCourse.images.length==0){ return; }

			var regex=/[-:\s]/g;
			var parts=TimeCourse.images[0].imageddatetime.split(regex);
			var timeZero= new Date(parts[0], parts[1]-1, parts[2], parts[3], parts[4], 0).getTime();
			timeZero=Math.floor(timeZero/1000);

			var scores=DropViewer.scores;
			var scoreIdsToScores=[];
			
			currentImage.timeCourseImages.each(function(img){
				var light=img.lighttype;
				var d=document.createElement("div");
				d.onmouseover=function(){ TimeCourse.showTimePointImage(d); }
				d.onmouseout=function(){ TimeCourse.unshowTimePointImage(d); }
				d.onclick=function(){ TimeCourse.setCurrentImage(d, true); }
				d.image=img;
				d.className="timepoint tc_"+light;
				if(img.id==DropViewer.images[DropViewer.currentIndex].id){
					d.className+=" currenttimepoint";
				}
				var i=document.createElement("img");
				i.src=img.thumbnailurl;

				if(""!=img.latestcrystalscoreid){
					var scoreId=img.latestcrystalscoreid;
					if(!scoreIdsToScores["s"+scoreId]){
						scores.each(function(s){
							scoreIdsToScores["s"+s.id]=s;
						});
					}
					i.style.borderColor="#"+scoreIdsToScores["s"+scoreId]["color"];
					i.title="Score: "+scoreIdsToScores["s"+scoreId]["label"];
				}
				d.appendChild(i);

				parts=img.imageddatetime.split(regex);
				var offsetTime= new Date(parts[0], parts[1]-1, parts[2], parts[3], parts[4], 0).getTime();
				offsetTime=Math.floor(offsetTime/1000);
				var s=document.createElement("span");
				s.className="tc_dateoffset";
				s.innerText=ui.secondsToFriendlyUnits(offsetTime-timeZero);
				d.appendChild(s);
				var s2=document.createElement("span");
				s2.className="tc_actualdate";
				s2.innerText=ui.friendlyDate(img.imageddatetime);
				d.appendChild(s2);
				container.appendChild(d);
 				if(!$("tc_"+light)){
					var btn=document.createElement("div");
					btn.className="tc_lighttype tc_lighttypeactive";
					btn.id="tc_"+light;
					btn.innerText=light;
					btn.dataset.light=light;
					btn.onclick=function(){ TimeCourse.toggleLightTypeImages(btn) };
					btn.style.backgroundImage='url(/images/icons/lighttypes/'+light+'.png)';
					$("tc_lighttypes").appendChild(btn);
					TimeCourse.activeLightTypes.push(light);
				}
			});
		},

		toggleLightTypeImages:function(button){
			if($(button).hasClassName("tc_lighttypeactive")){
				$(button).removeClassName("tc_lighttypeactive");
				$$("#tc_images .tc_"+button.dataset.light).each(function(dv){
					//dv.style.display="none";
					dv.addClassName("suppressed");
				});
				TimeCourse.activeLightTypes=TimeCourse.activeLightTypes.without(button.dataset.light);
			} else {
				$(button).addClassName("tc_lighttypeactive");
				$$("#tc_images .tc_"+button.dataset.light).each(function(dv){
					//dv.style.display="block";
					dv.removeClassName("suppressed");
				});
				TimeCourse.activeLightTypes.push(button.dataset.light);
			}
			var curr=$("tc_images").down(".currenttimepoint");
			if(curr && curr.hasClassName("suppressed")){
				if(!TimeCourse.goToPrevious()){
					TimeCourse.goToNext();
				}
			}
		},

		/**
		 * Mouseover/out image swap
		 */
		showTimePointImage:function(elem){
			if(!elem.image){ return false; }
			elem.previousImage=$("currentimage").src;
			$("currentimage").src=elem.image.fullimageurl;
		},		
		unshowTimePointImage:function(elem){
			if(elem.hasClassName("currenttimepoint")){ return false; }
			if(!elem.previousImage){ return false; }
			$("currentimage").src=elem.previousImage;
		},
		
		/**
		 * Movie/navigation controls
		 */
		goToFirst:function(){
			TimeCourse.stopMovie();
			var first=$("tc_images").down("div.timepoint:not(.suppressed)");
			if(!first){ return false; }
			TimeCourse.setCurrentImage(first);
		},
		goToPrevious:function(){
			TimeCourse.stopMovie();
			return TimeCourse.previousImage();
		},
		goToNext:function(){
			TimeCourse.stopMovie();
			return TimeCourse.nextImage();
		},
		goToLast:function(){
			var active=$$("#tc_images div.timepoint:not(.suppressed)");
			if(!active || 0==active.length){ return false; }
			var last=active.last();
			TimeCourse.setCurrentImage(last);
		},
		playOrStop:function(){
			if(TimeCourse.movieInterval){
				TimeCourse.stopMovie();
			} else {
				TimeCourse.startMovie();
			}
		},
		startMovie:function(){
			var current=$("tc_images").down(".currenttimepoint");
			var next=current.next("div.timepoint:not(.suppressed)");
			if(!next){
				TimeCourse.goToFirst();
			}
			TimeCourse.movieInterval=window.setInterval(TimeCourse.nextImage,DropViewer.movieDelay*1000);
			$("tc_playstop").title="Stop";
			$("tc_playstop").addClassName("playing");
		},
		stopMovie:function(){
			window.clearInterval(TimeCourse.movieInterval);
			TimeCourse.movieInterval=null;
			$("tc_playstop").title="Play";
			$("tc_playstop").removeClassName("playing");
		},
		nextImage:function(){
			var current=$("tc_images").down(".currenttimepoint");
			var next=current.next("div.timepoint:not(.suppressed)");
			if(!next){
				TimeCourse.stopMovie();
				return false;
			}
			TimeCourse.setCurrentImage(next);
			return true;
		},
		previousImage:function(){
			var current=$("tc_images").down(".currenttimepoint");
			var prev=current.previous("div.timepoint:not(.suppressed)");
			if(!prev){
				TimeCourse.stopMovie();
				return false;
			}
			TimeCourse.setCurrentImage(prev);
			return true;
		},

		getCurrentImage:function(){
			return TimeCourse.images[TimeCourse.currentIndex];
		},
		
		setCurrentImage:function(container, dontScroll){
			if($("tc_images").down(".currenttimepoint")){
				$("tc_images").down(".currenttimepoint").removeClassName("currenttimepoint");
			}
			container.addClassName("currenttimepoint");
			$("currentimage").src=container.image.fullimageurl;
			if(!dontScroll){
				$("tc_images").scrollTop=container.offsetTop;
			}
			$("currentimage").dataset.micronshigh=container.image.pixelheight*container.image.micronsperpixely;
			Measure.setScale();
			return true;
		},

		
}



/**********************************************************************************************************
 * DROP INFORMATION PANEL
 **********************************************************************************************************/

var DropInfo={
		
	switchTo:function(){
		$$(".controlpanel").each(function(p){
			p.style.display="none";
		});
		currentPanel=DropInfo;
		DropInfo.render();
		return true;
	},
	switchFrom:function(){
		$("dropinfopanel").innerHTML='';
	},
	refresh:function(){
		if(!DropViewer.images[DropViewer.currentIndex].welldrop){
			DropInfo.getWellDropInfo(); //triggers render()
		} else {
			DropInfo.render();
		}
	},
	getWellDropInfo:function(){
		var wellDropId=DropViewer.images[DropViewer.currentIndex].welldropid;
		new Ajax.Request("/api/welldrop/"+wellDropId,{
			method:"get",
			onSuccess:DropInfo.getWellDropInfo_onSuccess,
			onFailure:DropInfo.getWellDropInfo_onFailure,
		});
	},
	getWellDropInfo_onSuccess:function(transport){
		//set response to current drop's "welldrop" property
		if(!AjaxUtils.checkResponse(transport)){ return false; }
		DropViewer.images[DropViewer.currentIndex].welldrop=transport.responseJSON;
		DropInfo.render();
	},
	getWellDropInfo_onFailure:function(transport){
		DropInfo.render();
	},
	render:function(){
		if(DropInfo!=currentPanel){ return false; }
		var out='';
		out+='<br/><h2>Protein</h2>';
		var wellDrop=DropViewer.images[DropViewer.currentIndex].welldrop;
		if(wellDrop && wellDrop.proteinname && wellDrop.proteinname!=""){
			out+=DropViewer.images[DropViewer.currentIndex].welldrop.proteinname;
			out+='<br/>Construct: '+DropViewer.images[DropViewer.currentIndex].welldrop.constructname;
		} else {
			out+='(Not set)'
		}

		out+='<br/><br/><h2>Well solution</h2>';
		var screen=DropViewer.screen;
		if(null==screen || !screen.conditions){
			out+='Screen: (none chosen)';
		} else {
			var img=DropViewer.images[DropViewer.currentIndex];
			var plateTypeCols=DropViewer.plateType.cols;
			var wellNumber=((img.row-1)*plateTypeCols)+(1*img.col);
			out+=screen.conditions[wellNumber-1].description;
			out+='<br/>Screen: <a href="/screen/'+screen.id+'">'+screen.name+'</a>';
		}
		
		out+='<br/><br/><h2>Protein solution</h2>';
		if(""==wellDrop.proteinconcentrationamount || ""==wellDrop.proteinconcentrationunit){
			out+='Protein concentration: (Not set)'
		} else {
			var unit=wellDrop.proteinconcentrationunit;
			unit=unit.replace("uL","&#181;L");
			unit=unit.replace("ug","&#181;g");
			out+='Protein concentration: '+wellDrop.proteinconcentrationamount + unit;
		}
		out+='<br/>';
		if(""==wellDrop.proteinbuffer){
			out+='Protein buffer: (Not set)'
		} else {
			out+='Protein buffer: '+wellDrop.proteinbuffer;
		}
		
		out+='<br/><br/><h2>Drop volumes</h2>';
		if(""==wellDrop.proteinsolutionamount || ""==wellDrop.proteinsolutionunit){
			out+='Protein solution: (Not set)'
		} else {
			var unit=wellDrop.proteinsolutionunit;
			unit=unit.replace("uL","&#181;L");
			unit=unit.replace("ug","&#181;g");
			out+='Protein solution: '+wellDrop.proteinsolutionamount + unit;
		}
		out+='<br/>';
		if(""==wellDrop.wellsolutionamount || ""==wellDrop.wellsolutionunit){
			out+='Well solution: (Not set)'
		} else {
			var unit=wellDrop.wellsolutionunit;
			unit=unit.replace("uL","&#181;L");
			unit=unit.replace("ug","&#181;g");
			out+='Well solution: '+wellDrop.wellsolutionamount + unit;
		}
			
		out+='<br/><br/><h2>Plate</h2>';
		out+='Type: <a href="/platetype/'+DropViewer.plateType.id+'">'+DropViewer.plateType.name+'</a>';
		out+='<br/>Barcode: <a href="/plate/'+data.plateid+'">'+data.platename+'</a>';
		
		out+='<br/><br/><h2>Incubation</h2>';
		out+='Imager: <a href="/imager/'+data.imagerid+'">'+data.imagerfriendlyname+'</a>';
		out+='<br/>Temperature: '+data.temperature+'&deg;C';
		
		$("dropinfopanel").innerHTML=out;
		$("dropinfopanel").style.display="block";
	},

}

/**********************************************************************************************************
 * CRYSTAL SELECTION PANEL
 **********************************************************************************************************/

var CrystalSelection={
		
	crystals:[],
	currentCrystalNumber:null,

	switchTo:function(){
		Measure.stop();
		$$(".controlpanel").each(function(p){
			p.style.display="none";
		});
		currentPanel=CrystalSelection;
		CrystalSelection.getCrystals();
		$("currentimage").observe("click", CrystalSelection.addCrystal);
		return true;
	},
	switchFrom:function(){
		$$(".xs_crosshair").each(function(ch){ 
			if("xs_crosshair"==ch.id){
				ch.style.display="none";
			} else {
				ch.remove(); 
			} 
		});
		$("currentimage").stopObserving("click", CrystalSelection.addCrystal);
		Measure.init();
		$("selectpanel").innerHTML='';
		CrystalSelection.currentCrystalNumber=null;
	},
	refresh:function(){
		if(CrystalSelection!=currentPanel){ return false; }
		var out='';
		var currentCrystal=null;
		if(1==CrystalSelection.crystals.length){
			CrystalSelection.currentCrystalNumber=CrystalSelection.crystals[0].numberindrop;
		}
		CrystalSelection.crystals.each(function(c){
			if(c.numberindrop==CrystalSelection.currentCrystalNumber){
				currentCrystal=c;
			}
		});
		CrystalSelection.redrawCrosshairs();
		if(null==CrystalSelection.currentCrystalNumber){
			out+='Click any of the crosshairs on the image to select it, or click anywhere on the image to select a new crystal';
		} else {	
			var moveToImageButton="";
			if($("currentimage").src.indexOf(currentCrystal.dropimageid)<1){
				moveToImageButton=' <input type="button" style="cursor:pointer" onclick="CrystalSelection.updateCrystalImageNumber()" id="movetocurrentimage" value="Move to current image" title="Crosshair was marked on an older image."/>';
			}
			out+='<h2>Crystal '+CrystalSelection.currentCrystalNumber+moveToImageButton+'</h2>';
			if(canUpdate){
				out+='Sample name for synchrotron:<br/>'+currentCrystal.prefix+'<span id="nameseparator">_</span><input type="text" id="crystalsuffix" placeholder="(user-defined part)" value="'+currentCrystal.suffix+'"/>';
				out+='<div id="crystalnotesandfiles">';
					out+='<div id="crystalfiles">Waiting for files...</div>';
					out+='<div id="crystalnotes">Waiting for notes...</div>';
				out+='</div>';
				out+='<form id="crystalfilesadd" method="post" action="/api/file">';
					out+='<input type="hidden" value="'+csrfToken+'" name="csrfToken">';
					out+='<input type="hidden" value="'+currentCrystal.id+'" name="parentid">';
					out+='<strong>Attach a file: </strong><input id="crystalnotesaddfile" type="file" name="file" onchange="ui.submitForm(this)" />';
				out+='</form>';
				out+='<div id="crystalnotesadd">';
					out+='<textarea id="crystalnotesaddtext" placeholder="Add a note..."></textarea>';
					out+='<br/><input id="crystalnotesaddbutton" type="button" value="Save note" />';
				out+='</div>';
				
			} else {
				out+='Sample name for synchrotron:<br/>'+currentCrystal.prefix+'<span id="nameseparator">_</span>'+currentCrystal.suffix;
				out+='<div id="crystalnotesreadonly">';
					out+='<div id="crystalfiles">Waiting for files...</div>';
					out+='<div id="crystalnotes">Waiting for notes...</div>';
				out+='</div>';
			}
		}
		$("selectpanel").innerHTML=out;
		if($("nameseparator") && (""==currentCrystal.suffix || !currentCrystal.suffix)){
			$("nameseparator").style.visibility="hidden";
		}
		$("selectpanel").style.display="block";
		//get all the crystals, plot them
		$("xs_crosshair").style.display="block";
		if(null!=CrystalSelection.currentCrystalNumber){
			CrystalSelection.crystals.each(function(c){
				if(c.numberindrop==CrystalSelection.currentCrystalNumber){
					CrystalSelection.getCrystalNotes(c);
					CrystalSelection.getCrystalFiles(c);
				}
			});
		}
		if($("crystalnotesaddbutton")){
			$("crystalnotesaddtext").focus();
			$("crystalnotesaddbutton").observe("click",CrystalSelection.addCrystalNote);
		}
		if($("crystalsuffix")){
			$("crystalsuffix").observe("keyup",CrystalSelection.updateCrystalSuffix);			
		}
		if($("crystalnotesaddfile")){
			$("crystalnotesaddfile").options={ afterSuccess:CrystalSelection.addCrystalFile_onSuccess };
		}
	},

	redrawCrosshairs:function(){
		$$(".xs_crosshair").each(function(x){
			if("xs_crosshair"==x.id){ 
				x.style.display="none";
			} else {
				x.remove();
			}
		});
		CrystalSelection.crystals.each(function(c){
			var xh=CrystalSelection.writeCrosshair(c);
		});
	},
	
	writeCrosshair: function(crystal){
		var img=DropViewer.getCurrentImage();
		var newCrosshair=$("xs_crosshair").clone(true);
		var scale=$("currentimage").getHeight() / img.pixelheight;
		var offset=25;
		newCrosshair.id="crystal"+crystal.numberindrop;
		var pixelx=(crystal.pixelx * scale)-offset;
		var pixely=(crystal.pixely * scale)-offset;
		newCrosshair.style.top=pixely+"px";
		newCrosshair.style.left=pixelx+"px";
		newCrosshair.style.display="block";
		$("xs_crosshair").up().appendChild(newCrosshair);
		newCrosshair.observe("click",CrystalSelection.showCrystal);
		newCrosshair.crystal=crystal;
		if(CrystalSelection.currentCrystalNumber==crystal.numberindrop){
			newCrosshair.addClassName("xs_currentcrosshair");
		} else if($("currentimage").src.indexOf(crystal.dropimageid)<1){
			newCrosshair.addClassName("xs_crosshairfromdifferentimage");
			newCrosshair.title="Crystal was marked on a different image of this drop";
		}
		return newCrosshair;
	},
	
	updateCrystalSuffix:function(){
		if(window.suffixTimer){ window.clearTimeout(window.suffixTimer); }
		window.suffixTimer=window.setTimeout(CrystalSelection._doUpdateCrystalSuffix, 500);
		$("crystalsuffix").value=$("crystalsuffix").value.trim();
		if(""==$("crystalsuffix").value){ 
			$("nameseparator").style.visibility="hidden";
		} else {
			$("nameseparator").style.visibility="visible";
		}
	},
	_doUpdateCrystalSuffix:function(){
		if($("crystalsuffix").hasClassName("updating")){ return false; }
		var currentCrystal;
		CrystalSelection.crystals.each(function(c){
			var xh=CrystalSelection.writeCrosshair(c);
			if(c.numberindrop==CrystalSelection.currentCrystalNumber){
				currentCrystal=c;
			}
		});
		if(!currentCrystal){ return false; }
		$("crystalsuffix").addClassName("updating");
		new Ajax.Request('/api/crystal/'+currentCrystal.id,{
			method:'patch',
			parameters:{ 
				csrfToken:csrfToken,
				suffix:$("crystalsuffix").value
			},
			onSuccess:CrystalSelection.updateCrystalSuffix_onSuccess,
			onFailure:CrystalSelection.updateCrystalSuffix_onFailure,
		});
	},
	updateCrystalSuffix_onSuccess:function(transport){
		$("crystalsuffix").removeClassName("updating");
		if(!AjaxUtils.checkResponse(transport)){ return false; }
		var updated=transport.responseJSON.updated;
		CrystalSelection.crystals.each(function(c){
			if(c.id==updated.id){
				c.name=updated.name;
				c.suffix=updated.suffix;
			}
		});
		
	},
	updateCrystalSuffix_onFailure:function(transport){
		$("crystalsuffix").removeClassName("updating");
		AjaxUtils.checkResponse(transport);
	},
	
	updateCrystalImageNumber:function(){
		var crystalId=0;
		var imageId=DropViewer.images[DropViewer.currentIndex].id;
		CrystalSelection.crystals.each(function(xtal){
			if(xtal.numberindrop==CrystalSelection.currentCrystalNumber){
				crystalId=xtal.id;
			}
		});
		if(0==crystalId){ alert("Could not find crystal ID"); return false; }
		$("movetocurrentimage").up("h2").addClassName("updating");
		new Ajax.Request('/api/crystal/'+crystalId,{
			method:'patch',
			parameters:{
				csrfToken:csrfToken,
				dropimageid:imageId
			},
			onSuccess:CrystalSelection.updateCrystalImageNumber_onSuccess,
			onFailure:CrystalSelection.updateCrystalImageNumber_onFailure,
		});
	},
	updateCrystalImageNumber_onSuccess:function(transport){
		var updated=transport.responseJSON.updated;
		CrystalSelection.crystals.each(function(xtal){
			if(xtal.id==updated.id){
				xtal.dropimageid=updated.dropimageid;
			}
		});
		$("movetocurrentimage").up("h2").removeClassName("updating");
		$("movetocurrentimage").remove();
	},
	updateCrystalImageNumber_onFailure:function(transport){
		$("movetocurrentimage").up("h2").removeClassName("updating");
		AjaxUtils.checkResponse(transport);
	},
	
	showCrystal:function(evt){
		var xh=evt.target;
		while(!xh.crystal){
			xh=xh.up();
		}
		CrystalSelection.currentCrystalNumber=xh.crystal.numberindrop;
		CrystalSelection.refresh();
	},
	
	getCrystals:function(){
		var img=DropViewer.getCurrentImage();
		if(!img.crystals){
			new Ajax.Request('/api/welldrop/'+img.welldropid+'/crystal',{
				method:'get',
				onSuccess:CrystalSelection.getCrystals_onSuccess,
				onFailure:CrystalSelection.getCrystals_onFailure,
			});
		} else {
			CrystalSelection.crystals=img.crystals;
			CrystalSelection.refresh();
		}
	},
	getCrystals_onSuccess:function(transport){
		var img=DropViewer.getCurrentImage();
		img.crystals=transport.responseJSON.rows;
		CrystalSelection.crystals=transport.responseJSON.rows;
		CrystalSelection.refresh();
	},
	getCrystals_onFailure:function(transport){
		if(404==transport.status){
			CrystalSelection.crystals=[];
			CrystalSelection.refresh();
			return;
		}
		alert("Could not retrieve list of crystals for this well.");
	},


	getCrystalFiles:function(crystal){
		new Ajax.Request('/api/crystal/'+crystal.id+'/file',{
			method:'get',
			onSuccess:CrystalSelection.getCrystalFiles_onSuccess,
			onFailure:CrystalSelection.getCrystalFiles_onFailure,
		})
	},
	getCrystalFiles_onSuccess:function(transport){
		AjaxUtils.checkResponse(transport);
		var out='';
		out+='<h3>Files</h3>';
		transport.responseJSON.rows.each(function(file){
			out+='<a href="/file/'+file.id+'" target="_blank">'+file.filename+'</a><br/>';
		});
		out+='<hr/>';
		$("crystalfiles").innerHTML=out;
	},
	getCrystalFiles_onFailure:function(transport){
		if(404==transport.status){ 
			$("crystalfiles").innerHTML='';
			//$("crystalfiles").innerHTML='(No files)';
			return;
		}
		alert("Could not retrieve files for this crystal.");
	},


	
	getCrystalNotes:function(crystal){
		new Ajax.Request('/api/crystal/'+crystal.id+'/note',{
			method:'get',
			onSuccess:CrystalSelection.getCrystalNotes_onSuccess,
			onFailure:CrystalSelection.getCrystalNotes_onFailure,
		})
	},
	getCrystalNotes_onSuccess:function(transport){
		AjaxUtils.checkResponse(transport);
		var out='';
		transport.responseJSON.rows.each(function(note){
			out+='<h3><a href="/user/'+note.userid+'">'+note.username+'</a>, '+ui.friendlyDate(note.createtime)+'</h3>';
			out+=note.text+'<hr/><br/>';
		});
		$("crystalnotes").innerHTML=out;
	},
	getCrystalNotes_onFailure:function(transport){
		if(404==transport.status){ 
			$("crystalnotes").innerHTML='';
			//$("crystalnotes").innerHTML='(No notes)';
			return;
		}
		alert("Could not retrieve notes for this crystal.");
	},

	addCrystalNote: function(){
		if($("crystalnotesaddtext").hasClassName("updating")){ return false; }
		var parentId=0;
		CrystalSelection.crystals.each(function(c){
			if(c.numberindrop==CrystalSelection.currentCrystalNumber){
				parentId=c.id;
			}
		});
		if(0==parentId){ 
			alert("Could not determine crystal for note");
			return false;
		}
		var noteText=$("crystalnotesaddtext").value.trim();
		if(""==noteText){
			alert("Cannot add an empty note");
			return false;
		}
		$("crystalnotesaddtext").addClassName("updating");
		new Ajax.Request('/api/note',{
			method:'post',
			parameters:{
				csrfToken:csrfToken,
				parentid:parentId,
				text:noteText,
			},
			onSuccess:CrystalSelection.addCrystalNote_onSuccess,
			onFailure:CrystalSelection.addCrystalNote_onFailure,
		});
	},
	addCrystalNote_onSuccess: function(transport){
		CrystalSelection.refresh();
	},
	addCrystalNote_onFailure: function(transport){
		$("crystalnotesaddtext").removeClassName("updating");
		var err=transport.responseText;
		if(transport.responseJSON && transport.responseJSON.error){
			err=transport.responseJSON.error;
		}
		alert("Could not add note. The server said:\n\n"+err);
	},

	//No addCrystalFile - uses ui.submitForm
	addCrystalFile_onSuccess: function(transport){
		CrystalSelection.refresh();
	},
	addCrystalFile_onFailure: function(transport){
		$("crystalfilesaddfile").removeClassName("updating");
		var err=transport.responseText;
		if(transport.responseJSON && transport.responseJSON.error){
			err=transport.responseJSON.error;
		}
		alert("Could not add file. The server said:\n\n"+err);
	},

	
	addCrystal:function(evt){
		var crystalNumber=0;
		CrystalSelection.crystals.each(function(c){
			crystalNumber=Math.max(crystalNumber,c.numberindrop);
		});
		crystalNumber++;
		var img=DropViewer.getCurrentImage();
		var offset=$("currentimage").cumulativeOffset();
		var scale=$("currentimage").getHeight() / img.pixelheight;
		var crystalX=parseInt((evt.clientX-offset.left)/scale);
		var crystalY=parseInt((evt.clientY-offset.top)/scale);
		var img=DropViewer.getCurrentImage();
		var crystalName=DropViewer.getWellNameForImage(img)+'d'+img.dropnumber+'c'+crystalNumber;
		new Ajax.Request('/api/crystal',{
			method:'post',
			parameters:{
				csrfToken:csrfToken,
				numberindrop:crystalNumber,
				name:crystalName,
				pixelx:crystalX,
				pixely:crystalY,
				welldropid:img.welldropid,
				dropimageid:img.id,
				projectid:img.projectid
			},
			onSuccess:CrystalSelection.addCrystal_onSuccess,
			onFailure:CrystalSelection.addCrystal_onFailure
		});
	},
	addCrystal_onSuccess: function(transport){
		if(!transport.responseJSON|| !transport.responseJSON.created){
			return CrystalSelection.addCrystal_onFailure(transport);
		}
		CrystalSelection.crystals.push(transport.responseJSON.created);
		CrystalSelection.currentCrystalNumber=transport.responseJSON.numberindrop;
		CrystalSelection.refresh();
	},
	addCrystal_onFailure: function(transport){
		var err=transport.responseText;
		if(transport.responseJSON && transport.responseJSON.error){
			err=transport.responseJSON.error;
		}
		alert("Could not add crystal. The server said:\n\n"+err);
	},
	
	
	/* Movie controls when in crystals view */
	goToFirst:function(){
		DropViewer.goToFirst();
		CrystalSelection.switchFrom();
		CrystalSelection.switchTo();
	},
	goToLast:function(){
		DropViewer.goToLast();
		CrystalSelection.switchFrom();
		CrystalSelection.switchTo();
	},
	goToNext:function(){
		DropViewer.goToNext();
		CrystalSelection.switchFrom();
		CrystalSelection.switchTo();
	},
	goToPrevious:function(){
		DropViewer.goToPrevious();
		CrystalSelection.switchFrom();
		CrystalSelection.switchTo();
	},

	playOrStop:function(){
		if(DropViewer.movieInterval){
			CrystalSelection.stopMovie();
		} else {
			CrystalSelection.startMovie();
		}
	},

	startMovie:function(){
		$$(".xs_crosshair").each(function(ch){ 
			if("xs_crosshair"==ch.id){ 
				ch.style.display="none";
			} else {
				ch.remove(); 
			} 
		});
		DropViewer.startMovie();
	},

	stopMovie:function(){
		DropViewer.stopMovie();
		CrystalSelection.switchFrom();
		CrystalSelection.switchTo();
	},
	
	
	goUpOneRow:function(){
		DropViewer.goUpOneRow();
		CrystalSelection.switchFrom();
		CrystalSelection.switchTo();
	},
	goDownOneRow:function(){
		DropViewer.goDownOneRow();
		CrystalSelection.switchFrom();
		CrystalSelection.switchTo();
	},

	
}



/**********************************************************************************************************
 * MEASUREMENT 
 **********************************************************************************************************/

var Measure={

	suppressed:true,
	
	init:function(){
		$("imagepane").observe("mouseover", Measure.mouseOver);
		$("imagepane").observe("mousemove", Measure.mouseMove);
		$("imagepane").observe("mouseout",  Measure.mouseOut);
	},
			
	stop:function(){
		$("imagepane").stopObserving("mouseover", Measure.mouseOver);
		$("imagepane").stopObserving("mousemove", Measure.mouseMove);
		$("imagepane").stopObserving("mouseout",  Measure.mouseOut);
	},
			
	setScale:function(){
		var micronsHigh=$("currentimage").dataset.micronshigh;
		var pixelHeight=$("currentimage").getHeight();
		var screenMicronsPerPixel=micronsHigh/pixelHeight;
		$("dv_measure").style.height=Math.round(2000/screenMicronsPerPixel)+"px";
		$("dv_measure").style.width =Math.round(2000/screenMicronsPerPixel)+"px";
	},
	mouseOver:function(){
		$("dv_measure").style.display="block";
		Measure.mouseMove();

	},
	mouseMove:function(evt){
		if(!evt){ return false; }
		var absX=evt.pointer().x;
		var absY=evt.pointer().y;
		var imgPos=$("currentimage").getBoundingClientRect();
		var h=$("dv_measure").getHeight();
		var x=absX-imgPos.left;
		var y=absY-imgPos.top;
		$("dv_measure").style.left=(x)-(h/2)+"px";
		$("dv_measure").style.top=(y)-(h/2)+"px";
	},
	mouseOut:function(){
		$("dv_measure").style.display="none";
	},
	
	toggleVisibility:function(){
		if(Measure.suppressed){
			Measure.suppressed=false;
			Measure.init();
			Measure.mouseOver();
		} else {
			Measure.suppressed=true;
			Measure.mouseOut();
			Measure.stop();
		}
	},
	
}








/**********************************************************************************************************
 * DROP NAVIGATION
 **********************************************************************************************************/

var DropNav={

	isOpen:false,
		
	onclose:function(){
		DropNav.isOpen=false;
		$("modalBox").removeClassName("dropviewer");
		document.stopObserving("keyup",DropNav.navigateToDropOnEnterKey);
		return true;
	},

	open:function(){
		var pt=DropViewer.plateType;
		if(!pt){ alert("No plate type info, cannot show navigation"); return false; }
		DropNav.isOpen=true;
		ui.modalBox({
			title:"Drop navigation",
			content:"Just a moment...",
			onclose:DropNav.onclose
		});
		window.setTimeout(DropNav._doOpen,50);
	},
	_doOpen:function(){
		$("modalBox").addClassName("dropviewer");
		var pt=DropViewer.plateType;
		var dropCells=new Array();
		var dm=pt.dropmapping.split(",");
		var dmRows=dm.length;
		var dmCols=dm[0].length;
		var wellHeight=100/dmRows;
		var wellWidth=100/dmCols;
		for(r=0;r<dm.length;r++){
			var row=dm[r];
			for(c=0;c<row.length;c++){
				var pos=row[c];
				if(!isNaN(pos)){
					dropCells.push({
						dropNum:pos,
						height:wellHeight-2+"%",
						width:wellWidth-2+"%",
						top:r*wellHeight+"%",
						left:c*wellWidth+"%"
					});
				}
			}
		}
		var numDropCells=dropCells.length;
		
		var out='<table id="dropnav"><tr><th>Drop</th><td>&nbsp;</td>';
		for(i=1;i<=pt.cols;i++){
			out+="<th>"+i+"</th>";
		}
		out+="</tr>";
		for(r=1;r<=pt.rows;r++){
			out+="<tr>";
			if(r<=pt.subs){
				out+='<th class="droppicker droppickerempty" id="droppicker'+r+'" data-dropnumber="'+r+'" onclick="DropNav.selectDropNumber('+r+')">'+r+'</th>';
			} else {
				out+="<th>&nbsp;</th>";
			}
			out+="<th>"+DropViewer.rowLabels[r]+"</th>";
			for(c=1;c<=pt.cols;c++){
				out+='<td id="nav_'+r+'_'+c+'" data-row="'+r+'" data-col="'+c+'" class="dropnav_well empty" style="">';
				out+="</td>";
			}
			out+="</tr>";
		}
		out+="</tr></table>";
		
		$("modalBox").down(".boxbody").innerHTML=out;
		$("modalBox").down(".boxbody").down("tr").select("td,th").each(function(cell){
			cell.style.width=cell.getHeight()+"px";
		});
		var counter=0;
		DropViewer.images.each(function(img){
			var td=$("nav_"+img.row+"_"+img.col);
			td.removeClassName("empty");
			
			var dv=document.createElement("div");
			dv.id="dropnav_"+img.row+"_"+img.col+"_"+img.dropnumber;
			dv.className="dropnav_dropimage dropnav_drop"+img.dropnumber;
			dv.style.display="none";
			dv.dataset.row=img.row;
			dv.dataset.col=img.col;
			dv.dataset.dropnumber=img.dropnumber;
			dv.dataset.dropindex=counter;
			dv.dataset[img.lighttype.toLowerCase()+'image']=img.thumbnailurl;
			var score=$("score"+img.latestcrystalscoreid);
			if(score){
				dv.style.backgroundColor=score.style.borderColor;
			}
			var im=document.createElement("img");
			im.src=img.thumbnailurl;
			dv.appendChild(im);
			var lbl=document.createElement("div");
			lbl.addClassName("dropnav_comparelabel")
			lbl.innerHTML="Drop "+img.dropnumber;
			dv.appendChild(lbl);
			td.appendChild(dv);
			dv.observe("click",function(){ DropNav.goToDrop(dv); });
			dv.observe("mouseover",function(){ DropNav.startCompareDrops(dv); });
			dv.observe("mouseout",function(){ DropNav.stopCompareDrops(dv); });
			$("droppicker"+img.dropnumber).removeClassName("droppickerempty");
			counter++;
		});
		var currentImage=DropViewer.images[DropViewer.currentIndex];
		$("dropnav_"+currentImage.row+"_"+currentImage.col+"_"+currentImage.dropnumber).up("td").addClassName("dropnav_wellcurrent");
		document.observe("keyup", DropNav.navigateToDropOnEnterKey);
		window.setTimeout(function(){
			DropNav.selectDropNumber(currentImage.dropnumber);
			DropNav.setOtherLightTypeImages();
		},50);
	},
	
	otherImagingSession:null,
	setOtherLightTypeImages:function(){
		if(!DropNav.otherImagingSession || !DropNav.otherImagingSession.dropimages){
			window.setTimeout(DropNav.getOtherImagingSession,50);
			return;
		}
		if(0==DropNav.otherImagingSession.dropimages.length){ return false; }
		var lighttype=DropNav.otherImagingSession.dropimages[0].lighttype.toLowerCase();
		DropNav.otherImagingSession.dropimages.each(function(di){
			$("dropnav_"+di.row+"_"+di.col+"_"+di.dropnumber).dataset[lighttype+"image"]=di.thumbnailurl;
		});
		var th=$("dropnav").down("th");
		th.style.background="url(/images/icons/"+theme+"theme/switchlight.png) no-repeat center";
		th.style.cursor="pointer";
		th.innerHTML="";
		th.title="Switch between visible and UV images";
		th.onclick=DropNav.toggleLightTypeImages;
	},
	
	/**
	 * Examine the images currently shown in the drop nav pane, and retrieve an
	 * imaging session with the opposite light type (visible vs UV).
	 */
	getOtherImagingSession:function(){
		var otherLightType=false;
		var dv=$("dropnav").down("[data-visibleimage]");
		if(dv){ 
			otherLightType="UV";
			currentLightType="Visible";
		} else {
			dv=$("dropnav").down("[data-uvimage]");
			if(dv){ 
				otherLightType="Visible";
				currentLightType="UV";
			}
		}
		if(!otherLightType){ return false; }
		new Ajax.Request("/api/imagingsession/plateid/"+data['plateid']+"/lighttype/"+otherLightType+"?sortby=imageddatetime&sortdescending=yes",{
			method:"get",
			onSuccess: DropNav.getOtherImagingSession_onSuccess,
			onFailure: DropNav.getOtherImagingSession_onFailure,
		});
	},
	getOtherImagingSession_onSuccess:function(transport){
		//take the first one in transport.responseJSON.rows - was sorted latest first
		DropNav.otherImagingSession=transport.responseJSON.rows[0];
		DropNav.getOtherImagingSessionImages();
	},
	getOtherImagingSession_onFailure:function(transport){
		//nothing to do, just swallow it
	},
	getOtherImagingSessionImages:function(){
		new Ajax.Request("/api/imagingsession/"+DropNav.otherImagingSession.id+"/dropimage?pagesize=5000",{
			method:"get",
			onSuccess: DropNav.getOtherImagingSessionImages_onSuccess,
			onFailure: DropNav.getOtherImagingSessionImages_onFailure,
		});
	},
	getOtherImagingSessionImages_onSuccess:function(transport){
		DropNav.otherImagingSession.dropimages=transport.responseJSON.rows;
		DropNav.setOtherLightTypeImages();
	},
	getOtherImagingSessionImages_onFailure:function(transport){
		//nothing to do, just swallow it
	},

	currentLightType:"Visible",
	toggleLightTypeImages:function(){
		currentLightType= ("Visible"==currentLightType) ? "UV" : "Visible";
		var lt=currentLightType.toLowerCase();
		$$(".dropnav_dropimage").each(function(dv){
			dv.style.border="green"
			dv.down("img").src=dv.dataset[lt+"image"];
		});
	},
	
	startCompareDrops:function(dropDiv){
		//IE is "special" so we have to handle it "specially". Greeeeaaaaat....
		var isIE=/Trident\/|MSIE/.test(window.navigator.userAgent);
		$("modalBox").absolutize();
		var tr=dropDiv.up("tr");
		var tbl=tr.up("table");
		var cd=document.createElement("div");
		cd.id="dropnav_compare";
		cd.innerHTML=""+dropDiv.up("td").innerHTML.replace(' id="dropnav_', ' id="dropnavcompare_');
		cd.setStyle({ position:"absolute" });
		$("modalBox").down(".boxbody").appendChild(cd);
		cd.select(".dropnav_dropimage").each(function(di){
			di.setStyle({ display:"inline-block", height:"auto", width:"auto" });
		});
		Element.clonePosition(cd,dropDiv, { setWidth:false, setHeight:false, offsetLeft:-1, offsetTop:dropDiv.getHeight()+3 });
		var w1=cd.getWidth();
		var w2=dropDiv.up("tr").getWidth();
		var w3=dropDiv.getWidth();
		var l1=cd.cumulativeOffset().left;
		var l2=dropDiv.up("tr").cumulativeOffset().left;
		var l3=dropDiv.cumulativeOffset().left;
		if(l1+w1>=l2+w2){
			Element.clonePosition(cd,dropDiv.up("tr"), { setWidth:false, setHeight:false, setTop:false, offsetLeft:(w2-w1)-2 });
		}
		var lastCell=$$("#dropnav td").last();
		tableBottom=lastCell.cumulativeOffset().top+lastCell.getHeight();
		compareBottom=cd.cumulativeOffset().top+cd.getHeight();
		if(compareBottom>tableBottom){
			Element.clonePosition(cd,dropDiv.up("tr"), { setWidth:false, setHeight:false, setLeft:false, offsetTop:-(cd.getHeight()+2) });
		}
	},
	
	stopCompareDrops:function(){
		if($("dropnav_compare")){ $("dropnav_compare").remove(); }
	},
	
	selectDropNumber:function(dropNumber){
		//IE is "special" so we have to handle it "specially". Greeeeaaaaat....
		var isIE=/Trident\/|MSIE/.test(window.navigator.userAgent);
		$("modalBox").select("div.dropnav_dropimage").each(function(dv){
			dv.style.display="none";
		});
		$("modalBox").select("div.dropnav_drop"+dropNumber).each(function(dv){
			dv.setStyle({ display:"block" });
			if(isIE){
				var wrapper=dv.up();
				var bw=parseInt(wrapper.currentStyle.borderTopWidth);
				dv.style.height=wrapper.getHeight()-(2*bw)+"px";
				dv.style.width=wrapper.getWidth()-(2*bw)+"px";
			}
		});
		$("modalBox").select("th.droppicker").each(function(dv){
			dv.removeClassName("droppickercurrent");
		});
		$("droppicker"+dropNumber).addClassName("droppickercurrent");
	},
	
	navigateToDropOnEnterKey:function(evt){
		if(13!=evt.keyCode || !$("modalBox")){ return; }
		var selectedDrop=$("modalBox").down("div.dropnav_wellcurrent");
		if(selectedDrop){ DropNav.goToDrop(selectedDrop); }
	},
	
	goToDrop:function(dropElement){
		switchPanels($("viewbutton_dv"));
		DropViewer.setCurrentImage(dropElement.dataset.dropindex);
		ui.closeModalBox();
	},
	
	goToFirst:function(){
		DropNav.goToIndexInNavWindow(0);
	},
	goToPrevious:function(byDrop){
		if(byDrop || KeyboardShortcuts.ctrlPressed){
			DropNav.moveInNavWindow(0,0,-1);
		} else {
			DropNav.goLeftOneColumn();
		}
	},
	goToNext:function(byDrop){
		if(byDrop || KeyboardShortcuts.ctrlPressed){
			DropNav.moveInNavWindow(0,0,1);
		} else {
			DropNav.goRightOneColumn();
		}
	},
	goToLast:function(){
		DropNav.goToIndexInNavWindow(DropViewer.images.length-1);
	},

	goLeftOneColumn:function(){
		DropNav.moveInNavWindow(0,-1,0);
	},
	goRightOneColumn:function(){
		DropNav.moveInNavWindow(0,1,0);
	},
	goUpOneRow:function(){
		DropNav.moveInNavWindow(-1,0,0);
	},
	goDownOneRow:function(){
		DropNav.moveInNavWindow(1,0,0);
	},

	moveInNavWindow:function(rowDelta, colDelta, dropDelta){
		//what's highlighted now?
		var highlightedWell=$("modalBox").down("td.dropnav_wellcurrent");
		var highlightedDrop=$("modalBox").down("th.droppickercurrent");
		if(!highlightedWell || !highlightedDrop){ return false; }
		var oldRow=parseInt(highlightedWell.dataset.row);
		var oldCol=parseInt(highlightedWell.dataset.col);
		var oldDrop=parseInt(highlightedDrop.dataset.dropnumber);
		var toHighlight=null;
		var newDrop=null;
		if(0!=dropDelta){
			var oldIndex=parseInt($("dropnav_"+oldRow+"_"+oldCol+"_"+oldDrop).dataset.dropindex);
			DropNav.goToIndexInNavWindow(oldIndex+dropDelta);
			return;
		} else if(0!=rowDelta){
			var newRow=oldRow+rowDelta;
			toHighlight=$("dropnav_"+newRow+"_"+oldCol+"_"+oldDrop);
			if(!toHighlight){ return; }
			DropNav.goToIndexInNavWindow(toHighlight.dataset.dropindex);
			return;
		} else if(-1==colDelta){
			var prev=highlightedWell.previous("td.dropnav_well");
			if(!prev){
				var prevRow=highlightedWell.up("tr").previous("tr");
				if(prevRow){ prev=prevRow.select("td.dropnav_well").last(); }
			}
			if(prev){ 
				newDrop=prev.down(".dropnav_drop"+oldDrop);
				if(newDrop){
					DropNav.goToIndexInNavWindow(newDrop.dataset.dropindex); 
				}
			}
		} else if(1==colDelta){
			var next=highlightedWell.next("td.dropnav_well");
			if(!next){
				var nextRow=highlightedWell.up("tr").next("tr");
				if(nextRow){ next=nextRow.down("td.dropnav_well"); }
			}
			if(next){ 
				newDrop=next.down(".dropnav_drop"+oldDrop);
				if(newDrop){
					DropNav.goToIndexInNavWindow(newDrop.dataset.dropindex); 
				}
			}
		}
	},

	goToIndexInNavWindow:function(index){
		var img=DropViewer.images[index];
		if(!img){ return; }
		var dropDiv=$("dropnav_"+img.row+"_"+img.col+"_"+img.dropnumber);
		if(!dropDiv){ return; }
		$$(".dropnav_wellcurrent").each(function(elem){ elem.removeClassName("dropnav_wellcurrent")});
		dropDiv.up("td").addClassName("dropnav_wellcurrent");
		dropDiv.addClassName("dropnav_wellcurrent");
		DropNav.selectDropNumber(dropDiv.dataset.dropnumber);
		DropNav.stopCompareDrops();
		DropNav.startCompareDrops(dropDiv);
	},
	
}



/**********************************************************************************************************
 * OTHER FUNCTIONS
 **********************************************************************************************************/

function togglePageConfig(){
	var handedness=Cookies.get("dropviewer.handedness");
	var bb=ui.modalBox({ 
		title:'Configure the drop viewer',
		content:''
	});	
	var handednessDiv=document.createElement("div");
	handednessDiv.id="dv_config_handedness";
	
	var lh=document.createElement("div");
	lh.id="dv_config_lh";
	var limg=document.createElement("img");
	limg.src='/images/icons/dropviewer/controlsleft.png';
	limg.alt='Controls on the left';
	lh.appendChild(limg);
	lh.onclick=function(){ setHandedness("left"); }
	if("left"==handedness){ lh.style.borderColor='#666'; }
	handednessDiv.appendChild(lh);
	
	var rh=document.createElement("div");
	rh.id="dv_config_rh";
	var rimg=document.createElement("img");
	rimg.src='/images/icons/dropviewer/controlsright.png';
	rimg.alt='Controls on the right';
	rh.appendChild(rimg);
	rh.onclick=function(){ setHandedness("right"); }
	if("right"==handedness){ rh.style.borderColor='#666'; }
	handednessDiv.appendChild(rh);

	bb.appendChild(handednessDiv);
}

function setHandedness(hand){
	if("left"==hand){
		$(document.body).addClassName("lefthanded");
		Cookies.set("dropviewer.handedness","left");
		if($("dv_config_handedness")){
			$("dv_config_lh").style.borderColor="#666";
			$("dv_config_rh").style.borderColor="transparent";
		}
	} else {
		$(document.body).removeClassName("lefthanded");
		Cookies.set("dropviewer.handedness","right");
		if($("dv_config_handedness")){
			$("dv_config_lh").style.borderColor="transparent";
			$("dv_config_rh").style.borderColor="#666";
		}	
	}	
	window.setTimeout(ui.closeModalBox,100);
}

var currentPanel;
function switchPanels(button){
	var id=button.id;
	var newObj;
	if(id=="viewbutton_dv"){ newObj=DropViewer; }
	if(id=="viewbutton_tc"){ newObj=TimeCourse; }
	if(id=="viewbutton_di"){ newObj=DropInfo; }
	if(id=="viewbutton_xs"){ newObj=CrystalSelection; }
	if(newObj==currentPanel){ return false; }
	currentPanel.switchFrom();
	newObj.switchTo();
	currentPanel=newObj;
	$$(".viewbutton").each(function(vb){
		vb.removeClassName("current");
		vb.down("img").src=vb.down("img").src.replace("_current.png",".png");
	});
	button.addClassName("current");
	button.down("img").src=button.down("img").src.replace(".png","_current.png");
}
