<?php 

set_time_limit(0);
setUpClassAutoLoader();

$installDir=rtrim($_SERVER['DOCUMENT_ROOT'],'/').'/install/';

$icebearSessionStarted=false;
$error=null;
$pass=0;

//Formulatrix database handles
$rm=null;
$ri=null;

if(!file_exists($installDir.'preinstalldone')){
	writePage("dopre",'Run the set-up script first');
	exit;
}

$installComplete=false;
if(file_exists($installDir.'installcomplete')){
	$installComplete=true;
	writePage("done");
	exit;
}

if(isset($_GET['page'])){
	if(isset($_GET['pass'])){
		$pass=$_GET['pass'];
	} else{
		$pass=0;
	}
	$pass++;
	writePage($_GET['page']);
	exit;
}

if(!isset($_POST) || !isset($_POST['fromtemplate'])){
	writePage("prestart");
	exit;
}

$template=$_POST['fromtemplate'];
switch($template){
	case 'dopre':
		/*
		 * The user has run the pre-install script, having failed to do so previously.
		 * Show them the requirements for installation.
		 */
		$template='prestart';
		break;
	case 'prestart':
		/*
		 * They read the instructions (hopefully).
		 * Show them the configuration of image store and backup paths.
		 */
		$template='stores';
		break;
	case 'stores':
		/*
		 * They gave us image store and backup details. Attempt to configure.
		 * If successful, show the MySQL database config options.
		 */
		try{
			setUpStorage();
			$template='database';
		} catch (Exception $e){
			$error=$e->getMessage();
		}
		break;
	case 'database':
		/*
		 * They gave us the database connection details.
		 * Attempt to create the database and table structure, and write a config file for future connections to the DB.
		 * Show the form for creating the admin account.
		 */
		try{
			setUpDatabase();
			$template='adminaccount';
		} catch (Exception $e){
			$error=$e->getMessage();
		}
		break;
	case 'adminaccount':
		/*
		 * They gve us the admin account username and password.
		 * Create the admin account, load reference projects and usergroups, load standard screens
		 * Ask whether they have an imaging system or not.
		 */
		try{
			createAdminAccount();
			$template='imagingsystem';
		} catch (Exception $e){
			$error=$e->getMessage();
			var_dump($e);
		}
		break;
	case 'imagingsystem':
		/*
		 * They told us whether they have an imaging system.
		 * - if not, there is nothing else to do, so jump to the end.
		 * - if Formulatrix, ask them for connection details.
		 */
		try{
			$system=$_POST['imagingsystem'];
			if('fx'==$system){
				$template='imagerdb';
			} else if('none'==$system){
				setUpManualImaging();
				$template='done';
			}
		} catch (Exception $e){
			$error=$e->getMessage();
		}
		break;
	case 'imagerdb':
		/*
		 * They told us about the Formulatrix database and image stores.
		 * Verify connection details, write a script for root (who will run it via cron later) to mount the network shares.
		 * Copy imagers and users from RockMaker to Icebear. Set importer start date.
		 * Show imager configuration.
		 */
		try{
			setUpFormulatrixImagerDatabase();
			writeRootScriptToMountFormulatrixImageStores($_POST['_isuser'],$_POST['_ispass']);
			$template='imagers';
		} catch (Exception $e){
			$error=$e->getMessage();
		}
		break;
	case 'imagers':
		/*
		 * They told us the temperature, friendly name, capacity, etc., of the imagers.
		 * Update Icebear.
		 * Show them the users form.
		 */
		try{
			updateImagers();
			$template='users';
		} catch (Exception $e){
			$error=$e->getMessage();
		}
		break;
	case 'users':
		/*
		 * User submitted the Users form. If no changes, proceed to supervisors,
		 * otherwise return to Users form for review/further work.
		 */
		try{
			updateUsers();
			$template='users';
			if(isset($_POST['next'])){
				$template='supervisors';
			}				
		} catch (Exception $e){
			$error=$e->getMessage();
		}
		break;
	case 'supervisors':
		/*
		 * User submitted the Supervisors form. If no changes, proceed to Done,
		 * otherwise return to Supervisors form for review/further work.
		 */
		try{
			setSupervisorIds();
			$template='supervisors';
			if(isset($_POST['next'])){
				$template='done';
			}				
		} catch (Exception $e){
			$error=$e->getMessage();
		}
		break;
	case 'done':
		/*
		 * The user's part in the installation is complete. We are waiting for the root cron job
		 * to mount image stores, etc., and write the "installcomplete" file to the install directory.
		 * 
		 */
		if(!isset($_POST['pass'])){ 
			$pass=0; 
		} else {
			$pass=$_POST['pass'];
		}
		$installDir=rtrim($_SERVER['DOCUMENT_ROOT'],'/').'/install/';
		if(isRootCronFinished()){
			touch($installDir.'installcomplete');
		} else if($pass>20){
			writePage('done','Waited for final clean-up but it never happened.');
		}
		header('Location: /install/?page=done&pass='.$pass);
		exit;
		break;
}

if(!empty($error)){
	writePage($template,$error);
} else {
	goToPage($template);
}
exit;

/* Functions only below here */

function goToPage($template){
	header("Location: /install/?page=".$template);
	exit;
}

function writePage($template,$error=null){
	global $pass, $installComplete;
	include_once 'templates/_header.php';
	include_once 'templates/'.$template.'.php';
	include_once 'templates/_footer.php';
	exit;
}
/**
 * Register a class autoloader for the Icebear classes.
 */
function setUpClassAutoLoader(){
	spl_autoload_register(function($className){
		$paths=array(
				'../classes/',
				'../classes/core/',
				'../classes/core/exception/',
				'../classes/core/authentication/',
				'../classes/core/interface/',
				'../classes/model/',
		);
		foreach($paths as $path){
			if(file_exists($path.$className.'.class.php')){
				include_once($path.$className.'.class.php');
			}
		}
	});
}


function startIcebearSession(){
	global $icebearSessionStarted;
	database::connect();
	session::init(new DummySession());
	session::set('isAdmin',true);
	$firstUser=database::queryGetOne('SELECT id,name FROM user ORDER BY id ASC', array());
	if($firstUser){
		session::set('userId', $firstUser['id']);	
	}
	$icebearSessionStarted=true;
}


/**
 * Configure Icebear live and backup storage
 */
function setUpStorage(){
	$liveStore=$_POST['livestore'];
	$backupStore=$_POST['backupstore'];
	$backupsToKeep=(int)($_POST['backupstokeep']);
	$liveStore=rtrim($liveStore,'/');
	$backupStore=rtrim($backupStore,'/');
	if(empty($liveStore)){ throw new Exception('Storage path must be specified'); }
	if(empty($backupStore) && !empty($backupsToKeep)){ throw new Exception('Both backup store and number of backups must be specified to configure backups'); }
	if(!empty($backupStore) && empty($backupsToKeep)){ throw new Exception('Both backup store and number of backups must be specified to configure backups'); }
	//allow C: for dev on Windows only
	if('/'!=substr($liveStore,0,1) && 'C:'!=substr($liveStore,0,2)){ throw new Exception('Storage path must be absolute (begin with /)'); }
	if(file_exists($liveStore.'/fileuploads')){ 
		throw new Exception('Storage path '.$liveStore.' already has Icebear subdirectories. Is there a previous installation here? Aborting.'); 
	}
	if(!file_exists($liveStore)){ throw new Exception('Either storage path '.$liveStore.' does not exist or the web server does not have permission to see it'); }
	if(!@mkdir($liveStore.'/fileuploads') || !@mkdir($liveStore.'/imagestore')){
		throw new Exception('Store path exists, but the web server cannot write to it');
	}
	if(!empty($backupStore)){
		//allow C: for dev on Windows only
		if('/'!=substr($backupStore,0,1) && 'C:'!=substr($backupStore,0,2)){ throw new Exception('Backup path must be absolute (begin with /)'); }
		if(1>$backupsToKeep){ throw new Exception('If specifying a backup path, number of backups to keep must be at least 1'); }
		if(!file_exists($backupStore)){ throw new Exception('Either backup path '.$backupStore.' does not exist or the web server does not have permission to see it'); }
		$imageBackups=$backupStore.'/imagestore';
		$fileBackups=$backupStore.'/fileuploads';
		$dbBackups=$backupStore.'/database';
		if(file_exists($backupStore.'/fileuploads')){ 
			throw new Exception('Backups path '.$backupStore.' already has Icebear subdirectories. Is there a previous installation here? Aborting.'); 
		}
		if(!@mkdir($imageBackups) || !@mkdir($fileBackups) || !@mkdir($dbBackups)){
			throw new Exception('Backup path exists, but the web server cannot write to it');
		}
	} else {
		$backupStore='';
		$imageBackups='';
		$fileBackups='';
		$dbBackups='';
	}
	$contents=implode(PHP_EOL,array(
			";for security:",
			";<?php exit(); __halt_compiler();",
			"",
			"core_databasebackupstokeep=$backupsToKeep",
			"core_databasebackup=$dbBackups",
			"core_filestorebackup=$fileBackups",
			"core_imagestorebackup=$imageBackups",
			"core_filestore=$liveStore/fileuploads",
			"core_imagestore=$liveStore/imagestore",
	));
	$file=@fopen(rtrim($_SERVER['DOCUMENT_ROOT'],'/').'/install/storepaths.ini', 'w');
	if(!$file || !@fwrite($file, $contents) || !@fclose($file)){
		throw new Exception("Could not write the storage paths to a temporary config file");
	}
	
}


/**
 * Set up Icebear database
 */
function setUpDatabase(){
	$host=$_POST['host'];
	$rootuser=$_POST['user'];
	$rootpass=$_POST['pass'];
	$dbname=$_POST['dbname'];
	$ibuser=$_POST['dbusername'];
	
	$db = new PDO("mysql:host=$host", $rootuser, $rootpass, array(PDO::MYSQL_ATTR_FOUND_ROWS => true));
	$db->setAttribute(PDO::ATTR_ERRMODE, PDO::ERRMODE_EXCEPTION);
	
	$stmt=$db->prepare('SHOW DATABASES');
	$stmt->execute(array());
	$result=$stmt->fetchAll();
	foreach($result as $row){
		if($row['Database']==$dbname){
			throw new Exception('A database called "'.$dbname.'" already exists');
		}
	}
	
	$ibpass='';
	$chars='bcdfghjklmnpqrstvwxyzBCDEFGHJKLMNPQRSTVWXYZ0123456789';
	$max = strlen($chars)-1;
	for ($i=0; $i<16; $i++) {
		$ibpass.=$chars[mt_rand(0, $max)];
	}

	$stmt=$db->prepare('SELECT User FROM mysql.user');
	$stmt->execute(array());
	$result=$stmt->fetchAll();
	foreach($result as $row){
		if($row['User']==$ibuser){
			throw new Exception('A user called "'.$ibuser.'" already exists');
		}
	}

	$db->beginTransaction();
	try{
		$queries=array(
			'CREATE USER "'.$ibuser.'"@"localhost" IDENTIFIED BY "'.$ibpass.'"',
			'CREATE DATABASE IF NOT EXISTS '.$dbname.' DEFAULT CHARACTER SET utf8 COLLATE utf8_bin;',
			'GRANT INSERT,DELETE,SELECT,UPDATE ON '.$dbname.'.* TO "'.$ibuser.'"@"localhost"',
			'USE '.$dbname.';'
		);
		foreach($queries as $query){
			$stmt=$db->prepare($query);
			$stmt->execute(array());
		}
		
		//Parse the database
		$sql = file_get_contents('icebeardatabase.sql');
		$db->exec($sql);
		
		$db->commit();
		
	} catch(Exception $e){
		$db->rollBack();
		throw new Exception("An error occurred while creating the Icebear database and user:\n".$e->getMessage());
	}
	
	writeConfigFile($host, $ibuser, $ibpass, $dbname);	

	//Lastly, read storage and backup paths from temporary file and set them in the config table
	$paths=parse_ini_file(rtrim($_SERVER['DOCUMENT_ROOT'],'/').'/install/storepaths.ini');
	if(!$paths){
		throw new Exception("Database was created, but couldn't get storage paths from temporary config file.");
	}
	$stmt=$db->prepare('UPDATE config SET value=:value WHERE name=:key');
	foreach($paths as $k=>$v){
		$stmt->execute(array(':key'=>$k, ':value'=>$v));
	}
	@unlink(rtrim($_SERVER['DOCUMENT_ROOT'],'/').'/install/storepaths.ini');

}

function writeConfigFile($host,$user,$pass,$dbname){
	$contents=implode(PHP_EOL,array(
			";for security:",
			";<?php exit(); __halt_compiler();",
			"",
			"dbType=mysql",
			"dbHost=$host",
			"dbUser=$user",
			"dbPass=$pass",
			"dbName=$dbname"
	));
	
	$file=@fopen('../conf/config.ini', 'w');
	if(!$file || !@fwrite($file, $contents) || !@fclose($file)){
		throw new Exception("Could not write the database details to a config file");
	}
	
}

/**
 * Set up Icebear admin account
 */

function createAdminAccount(){
	if($_POST['pass']!=$_POST['pass2']){
		throw new Exception('Password fields do not match');
	}
	
	try {

		$conf=parse_ini_file(realpath(__DIR__).'/../conf/config.ini');
		if(!$conf){ die('Could not read database connection details from config file'); }
		$db = new PDO('mysql:host='.$conf['dbHost'].';dbname='.$conf['dbName'], $conf['dbUser'], $conf['dbPass'], array(PDO::MYSQL_ATTR_FOUND_ROWS => true));
		$conf=null; //for security, do not keep the DB credentials
		$db->setAttribute(PDO::ATTR_ERRMODE, PDO::ERRMODE_EXCEPTION);
		
		$db->beginTransaction();
		
		$stmt=$db->prepare('INSERT INTO user(name, fullname, password, email, isactive) VALUES(:name, :fullname, :password, :email, :isactive)');
		$stmt->execute(array(
				':name'=>$_POST['user'],
				':password'=>password_hash($_POST['pass'], PASSWORD_BCRYPT),
				':fullname'=>'Icebear Administrator',
				':email'=>'bogus@null.null',
				':isactive'=>1
		));
		$userId=$db->lastInsertId();
		
		//create usergroups, etc.
		loadRefDataPart1($db, $userId);
		
		//put user into Admins group
		$stmt=$db->prepare('SELECT id FROM usergroup WHERE name="Administrators"');
		$stmt->execute(array());
		$grp=$stmt->fetch();
		$grpId=$grp['id'];
		
		$stmt=$db->prepare('INSERT INTO groupmembership(userid, usergroupid, isgroupadmin) VALUES(:userid, :groupid, 1)');
		$stmt->execute(array(':userid'=>$userId, ':groupid'=>$grpId));
		
		//Populate their homepage with bricks
		$homepageBricks=array(
				array(1,1,'fi_oulu_PlateInspections'),
				array(2,1,'fi_oulu_Tasks'),
				array(3,1,'fi_oulu_DiskUsage'),
				array(1,2,'fi_oulu_Imagers'),
				array(2,2,'fi_oulu_AutoProvisionedUsers'),
				array(1,3,'fi_oulu_FirstBrick'),
		);
		foreach($homepageBricks as $b){
			$stmt=$db->prepare('SELECT id FROM homepagebrick WHERE name=:brickname');
			$stmt->execute(array(':brickname'=>$b[2]));
			$brk=$stmt->fetch();
			if(empty($brk)){ continue; }
			$brkId=$brk['id'];
			$stmt=$db->prepare('INSERT INTO homepageuserbrick(userid, homepagebrickid, row, col) VALUES(:userid,:brickid,:row,:col)');
			$stmt->execute(array(':userid'=>$userId, ':brickid'=>$brkId, ':row'=>$b[0], ':col'=>$b[1]));
		}
		
		$db->commit();
		
	} catch (Exception $e){
		$db->rollBack();
		throw new Exception($e->getMessage());
	}

	loadRefDataPart2();


}

/**
 * Configures Icebear for manual imaging
 */
 function setUpManualImaging(){
 	startIcebearSession();
	try {
 		database::begin();
 		config::set('imaging_allowmanual', 1);
 		config::set('fx_hasimagers', 0);
		//set has rigaku to false
 		database::commit();
	} catch (Exception $e){
		database::abort();
		throw new Exception($e->getMessage());
	}
 }

/**
 * Configures Formulatrix connection
 */

function setUpFormulatrixImagerDatabase(){
	startIcebearSession();
	database::begin();
	try {
 		config::set('imaging_allowmanual', 0);
		config::set('fx_hasimagers', 1);
		//set has rigaku to false
		foreach($_POST as $k=>$v){
			if(0===strpos($k,'fx_')){
				config::set($k, trim($v));
			}
		}
		database::commit();
		fxConnect();
		fxGetImagers();
		fxGetUsers();
	} catch (Exception $e){
		database::abort();
		throw new Exception($e->getMessage());
	}
}

function writeRootScriptToMountFormulatrixImageStores($diskuser,$diskpass){
	$root='/mnt/Formulatrix/';
	$imageStores=fxGetImageStores();
	$out='';
	$out.='touch '.rtrim($_SERVER['DOCUMENT_ROOT'],'/').'/install/rootcronstarted'.PHP_EOL;
	$out.='mkdir '.$root.PHP_EOL;
	$out.='rm '.$root.'.smbcredentials'.PHP_EOL;
	$out.='echo \'username='.$diskuser.'\' >> '.$root.'.smbcredentials'.PHP_EOL;
	$out.='echo \'password='.$diskpass.'\' >> '.$root.'.smbcredentials'.PHP_EOL;
	$out.='echo \'#Formulatrix image stores - added by Icebear installer\' >> /etc/fstab'.PHP_EOL;
	foreach($imageStores as $s){
		$s['Name']=preg_replace('/[^0-9a-zA-Z]/i','',$s['Name']);
		//mkdir
		$out.='mkdir '.$root.''.$s['Name'].PHP_EOL;
		//append to /etc/fstab
		$path=str_replace('\\', '/', $s['BasePath']);
		$out.='echo \''.$path.' '.$root.$s['Name'].' cifs ro,credentials='.$root.'.smbcredentials,iocharset=utf8,sec=ntlm 0 0\' >> /etc/fstab'.PHP_EOL;
	}
	$out.='mount -a'.PHP_EOL;
	
	$cron=array();
	
	//add importer to crontab
	$cron[]='# Icebear: Import plates, etc., from Formulatrix imagers.';
	$cron[]='# Run the importer every 5 minutes, rotate the logs nightly.';
	$cron[]='*/5 * * * * php '.rtrim($_SERVER['DOCUMENT_ROOT'],'/').'/formulatriximporter/importer.php -m5 -l2 >> /var/log/icebear/import.log';
	$cron[]='59 23 * * * '.rtrim($_SERVER['DOCUMENT_ROOT'],'/').'/formulatriximporter/rotatelogs.sh';
	
	//add backup to crontab
	$cron[]='# Icebear: Back up the database and WWW files if configured to do so.';
	$cron[]='# Run the backup nightly, rotate the logs once a week.';
	$cron[]='# Back up just after 00:00 but before first importer run, so backup date is correct and importer waits.';
	$cron[]='4 0 * * * php '.rtrim($_SERVER['DOCUMENT_ROOT'],'/').'/backup/backup.php -l2 >> /var/log/icebear/backup.log';
	$cron[]='58 23 * * * '.rtrim($_SERVER['DOCUMENT_ROOT'],'/').'/backup/rotatelogs.sh';

	foreach($cron as $c){
		$out.='( crontab -l | grep -v -F "'.$c.'" ; echo "'.$c.'" ) | crontab -'.PHP_EOL;
	}
	
	//remove this temporary script from the crontab
	$out.='( crontab -l | grep -v -F "rootinstalltasks.sh" ) | crontab - '.PHP_EOL;
	$out.='touch '.rtrim($_SERVER['DOCUMENT_ROOT'],'/').'/install/rootcrondone'.PHP_EOL;
	
	//dump to file
	$outfile=@fopen(rtrim($_SERVER['DOCUMENT_ROOT'],'/').'/install/rootinstalltasks.sh','w');
	if(!$outfile){ throw new Exception('Could not open temporary root script '.rtrim($_SERVER['DOCUMENT_ROOT'],'/').'install/rootinstalltasks.sh'.' for writing'); }
	fwrite($outfile, $out);
	fclose($outfile);
	chmod(rtrim($_SERVER['DOCUMENT_ROOT'],'/').'/install/rootinstalltasks.sh',0755);
}


/**
 * Connect to the RockMaker and RockImager databases and return true, or throw an exception.
 * @return boolean
 */
function fxConnect(){
	global $ri,$rm;
	$hostname=config::get('fx_dbhost');
	$port=config::get('fx_dbport');
	$username=config::get('fx_dbuser');
	$password=config::get('fx_dbpass');
	$rmdbname=config::get('fx_rmdbname');
	$ridbname=config::get('fx_ridbname');
	$drivers=PDO::getAvailableDrivers();
	$message='';

	if(in_array('dblib', $drivers)){
		try {
			$rm = new PDO ("dblib:host=$hostname:$port;dbname=$rmdbname",$username,$password);
			$rm->setAttribute( PDO::ATTR_ERRMODE, PDO::ERRMODE_EXCEPTION );
			$ri = new PDO ("dblib:host=$hostname:$port;dbname=$ridbname",$username,$password);
			$ri->setAttribute( PDO::ATTR_ERRMODE, PDO::ERRMODE_EXCEPTION );
			return true;
		} catch(Exception $e){
			//dblib: attempt failed
			$message.=PHP_EOL."Attempting with dblib gave: ".$e->getMessage();
		}
	} else if(in_array('sqlsrv', $drivers)){
		try {
			$rm = new PDO ("sqlsrv:Server=$hostname,$port; Database=$rmdbname",$username,$password);
			$rm->setAttribute( PDO::ATTR_ERRMODE, PDO::ERRMODE_EXCEPTION );
			$ri = new PDO ("sqlsrv:Server=$hostname,$port;Database=$ridbname",$username,$password);
			$ri->setAttribute( PDO::ATTR_ERRMODE, PDO::ERRMODE_EXCEPTION );
			return true;
		} catch(Exception $e){
			//sqlsrv: attempt failed
			$message.=PHP_EOL."Attempting with sqlsrv gave: ".$e->getMessage();
		}
	}
	$message.=PHP_EOL."Available PDO drivers: ".implode(', ',$drivers);
	throw new Exception("Could not connect to Formulatrix databases:".$message);
}

function fxGetImagers(){
	global $ri,$rm;
	if(null==$ri){ throw new Exception('call fxConnect before fxGetImagers'); }

	$stmt=$ri->prepare('SELECT Name FROM Robot');
	$stmt->execute(array());
	$imagers=$stmt->fetchAll(PDO::FETCH_ASSOC);
	foreach($imagers as $imager){
		$imagerName=$imager['Name'];
		$imagerType=explode('-', $imagerName)[0];
		$capacity=10000;
		switch($imagerType){
			case 'RI1':
				$capacity=1;
				break;
			case 'RI2':
				$capacity=2;
				break;
			default:
				$stmt=$ri->prepare('SELECT COUNT(Robot.Name) AS capacity 
						FROM Address, Hotel, Robot
						WHERE Address.HotelID=Hotel.ID 
							AND Address.RobotID=Robot.ID
							AND Hotel.IsPresent=1
						GROUP BY Robot.Name
						HAVING Robot.Name=:name
				');
				$stmt->execute(array(':name'=>$imagerName));
				$result=$stmt->fetch(PDO::FETCH_ASSOC);
				if(!$result){
					$capacity=10000;
				} else {
					$capacity=$result['capacity'];
				}
		}
		imager::create(array(
				'name'=>$imagerName,
				'friendlyname'=>$imagerName,
				'manufacturer'=>'Formulatrix',
				'temperature'=>20,
				'platecapacity'=>$capacity,
				'alertlevel'=>floor($capacity*75/100),
				'warninglevel'=>floor($capacity*90/100)
		));
	}
}

function fxGetImageStores(){
	global $ri,$rm;
	if(null==$rm){ throw new Exception('call fxConnect before fxGetImageStores'); }

	$stmt=$rm->prepare('SELECT Name, BasePath FROM ImageStore');
	$stmt->execute(array());
	$imageStores=$stmt->fetchAll(PDO::FETCH_ASSOC);
	return $imageStores;
}

function fxGetUsers(){
	global $ri,$rm;
	if(null==$rm){ throw new Exception('call fxConnect before fxGetUsers'); }
	$stmt=$rm->prepare("SELECT Name,EmailAddress FROM Users WHERE Name NOT IN('Administrator','FormulatrixAdmin','Default User')");
	$stmt->execute(array());
	$users=$stmt->fetchAll(PDO::FETCH_ASSOC);
	foreach($users as $user){
		$temporaryUsername='AUTO'.rand(10000000,99999999);
		while(user::getByName($temporaryUsername)){
			$temporaryUsername='AUTO'.rand(10000000,99999999);
		}
		createUser($temporaryUsername, $user['Name'] , $user['EmailAddress'], true, false, false);
	}
}

function createUser($name, $fullName, $email, $isActive, $isAdmin, $isSupervisor){
	$user=array(
			'name'=>$name,
			'fullname'=>$fullName,
			'password'=>'ChangeMe!',
			'email'=>$email,
			'isactive='=>$isActive
	);
	$user=user::create($user);
	$userId=$user['created']['id'];
	$adminGroup=usergroup::getByName('Administrators');
	$superGroup=usergroup::getByName('Supervisors');
	$adminGroupId=$adminGroup['id'];
	$superGroupId=$superGroup['id'];
	if($isAdmin){
		groupmembership::create(array('userid'=>$userId, 'usergroupid'=>$adminGroupId));
	}
	if($isSupervisor){
		groupmembership::create(array('userid'=>$userId, 'usergroupid'=>$superGroupId));
	}
	return $userId;
}

/**
 * Configure imagers
 */
function updateImagers(){
	startIcebearSession();
	try {
		database::begin();
		foreach($_POST as $k=>$v){
			if(0===strpos($k,'imager_')){
				$parts=explode('_',$k);
				$imagerId=$parts[1];
				$field=$parts[2];
				imager::update($imagerId, array($field=>$v));
			}
		}
		database::commit();
	} catch (Exception $e){
		database::abort();
		throw new Exception($e->getMessage());
	}
}

/**
 * Configure users
 */
function updateUsers(){
	global $createdUsers;
	startIcebearSession();
	try {
		database::begin();
		$users=array();
		$newUsers=array();
		foreach($_POST as $k=>$v){
			if(0===strpos($k,'user_')){
				$parts=explode('_',$k);
				$userId=$parts[1];
				$field=$parts[2];
				if('delete'==$field){
					if(isset($users['user'.$userId])){
						unset($users['user'.$userId]);
					}
					database::query(
						'DELETE FROM user WHERE id=:userid',
						array(':userid'=>$userId)
					);
					continue;
				}
				if(!isset($users['user'.$userId])){
					$users['user'.$userId]=array(
							'id'=>$userId,
							'admin'=>0,
							'active'=>0,
							'super'=>0,
							'name'=>'',
							'fullname'=>''
					);
				}
				if('isactive'==$field){
					$users['user'.$userId]['active']=1;
				} else if('isadmin'==$field){
					$users['user'.$userId]['admin']=1;
				} else if('issuper'==$field){
					$users['user'.$userId]['super']=1;
				} else if('name'==$field){
					$users['user'.$userId]['name']=$v;
				} else if('fullname'==$field){
					$users['user'.$userId]['fullname']=$v;
				} 
			} else if(0===strpos($k,'create_')){
				$parts=explode('_',$k);
				$userId=$parts[1];
				$field=$parts[2];
				if(!isset($newUsers['new'.$userId])){
					$newUsers['new'.$userId]=array(
							'admin'=>0,
							'active'=>0,
							'super'=>0,
							'name'=>'',
							'fullname'=>'',
							'email'=>''
					);
				}
				if('isactive'==$field){
					$newUsers['new'.$userId]['active']=1;
				} else if('isadmin'==$field){
					$newUsers['new'.$userId]['admin']=1;
				} else if('issuper'==$field){
					$newUsers['new'.$userId]['super']=1;
				} else if('name'==$field){
					$newUsers['new'.$userId]['name']=$v;
				} else if('fullname'==$field){
					$newUsers['new'.$userId]['fullname']=$v;
				} else if('email'==$field){
					$newUsers['new'.$userId]['email']=$v;
				} 
			}
		}
		$adminGroup=usergroup::getByName('Administrators');
		$superGroup=usergroup::getByName('Supervisors');
		$adminGroupId=$adminGroup['id'];
		$superGroupId=$superGroup['id'];
		foreach($users as $u){
			$uid=$u['id'];
			user::update($uid,array(
					'name'=>$u['name'],
					'fullname'=>$u['fullname'],
					'isactive'=>$u['active']
			));
			if(1==$u['admin']){
				if(!usergroup::userisingroup($adminGroupId, $uid)){
					groupmembership::create(array('userid'=>$uid, 'usergroupid'=>$adminGroupId));
				}
			} else {
				database::query(
					'DELETE FROM groupmembership WHERE userid=:userid AND usergroupid=:usergroupid',
					array(':userid'=>$uid, ':usergroupid'=>$adminGroupId)
				);
			}
			if(1==$u['super']){
				if(!usergroup::userisingroup($superGroupId, $uid)){
					groupmembership::create(array('userid'=>$uid, 'usergroupid'=>$superGroupId));
				}
			} else {
				database::query(
					'DELETE FROM groupmembership WHERE userid=:userid AND usergroupid=:usergroupid',
					array(':userid'=>$uid, ':usergroupid'=>$superGroupId)
				);
			}
		}
		foreach($newUsers as $n){
			if(""==$n['name']){ continue; }
			createUser($n['name'], $n['fullname'], $n['email'], $n['active'], $n['admin'], $n['super']);
			$createdUsers=true;
		}
		database::commit();
	} catch (Exception $e){
		database::abort();
		throw new Exception($e->getMessage());
	}
}

/**
 * Configure supervision
 */

function setSupervisorIds(){
	startIcebearSession();
	database::begin();
	try {
		foreach($_POST as $k=>$v){
			if(0===strpos($k,'user_')){
				$parts=explode('_',$k);
				$userId=$parts[1];
				$field=$parts[2];
				if('supervisorid'==$field){
					if(empty($v)){ $v=null; }
					user::update($userId, array('supervisorid'=>$v));
				}
			}
		}
		database::commit();
	} catch (Exception $e){
		database::abort();
		throw new Exception($e->getMessage());
	}
}

/**
 * Returns true and starts the Formulatrix importer if the root cron job written earlier has completed, otherwise returns false.
 */
function isRootCronFinished(){
	$installDir=rtrim($_SERVER['DOCUMENT_ROOT'],'/').'/install/';
	if(file_exists($installDir.'rootcrondone')){
		//If we get here, cron did its thing. Formulatrix image stores are mounted and the importer is in the root cron, so enable the importer.
		startIcebearSession();
		database::begin();
		if(1==config::get('fx_hasimagers')){
			config::set('fx_importerrunning',true);
		}
		database::commit();
		return true;	
	}	
	return false;
}


/**
 * Reference data
 */
function loadRefDataPart1($db, $adminId){ //db is a PDO handle
	
	
	//Projects
	$file=@fopen(__DIR__.'/refdata/project.csv','r');
	if(!$file){ throw new Exception('Could not open projects list for import'); }
	$keys=fgetcsv($file);
	$keys[]='owner';
	$placeholders=substr(str_repeat(',?', count($keys)), 1);
	$query='INSERT INTO project('.implode(',', $keys).') VALUES('.$placeholders.')';
	$stmt=$db->prepare($query);
	$values=fgetcsv($file);
	while(!empty($values)){
		$values[]=$adminId;
		$stmt->execute($values);
		$values=fgetcsv($file);
	}
	@fclose($file);
	
	//Usergroups
	$file=@fopen('refdata/usergroup.csv','r');
	if(!$file){ throw new Exception('Could not open usergroups list for import'); }
	$keys=fgetcsv($file);
	$placeholders=substr(str_repeat(',?', count($keys)), 1);
	$query='INSERT INTO usergroup('.implode(',', $keys).') VALUES('.$placeholders.')';
	$stmt=$db->prepare($query);
	$values=fgetcsv($file);
	while(false!==$values){
		$values=array_values($values);
		$stmt->execute($values);
		$values=fgetcsv($file);
	}
	@fclose($file);
}	
	
function loadRefDataPart2(){	
	startIcebearSession();
	
	try {

		database::begin();
		
		//Get the shared project, for later ref data
		$sharedProject=project::getByName(project::SHARED);
		if(!$sharedProject){ throw new Exception("Shared project does not exist. Cannot create screens."); }
		$sharedProjectId=$sharedProject['id'];
		
		//Standard screens
		$destinationDir=rtrim(config::get('core_filestore'),'/').'/files/project'.$sharedProjectId.'/';
		$backupDir='';
		if(!empty(config::get('core_filestorebackup'))){
			$backupDir=rtrim(config::get('core_filestorebackup'),'/').'/files/project'.$sharedProjectId.'/';
		}
		if(!file_exists($destinationDir)){
			if(!@mkdir($destinationDir, 0700, true)){
				throw new ServerException('Could not make project directory in filestore for shared project');
			}
		}
		$source="refdata/screen/";
		$sourceDir=dir($source);
		while(false!==($filename=$sourceDir->read())) {
			if(!preg_match('/.+_icebear\.csv$/',$filename)){ continue; }
			$screenName=str_replace('_icebear.csv','',$filename);
			$fileContent=file_get_contents($source.$filename);
			$request=array(
					'name'=>$screenName,
					'projectid'=>$sharedProjectId
			);
			$screen=screen::parseWellNameAndConditionToDatabase($fileContent,$request);
			$screenId=$screen['created']['id'];
			$request=array(
				'projectid'=>$sharedProjectId,
				'parentid'=>$screenId,
				'filename'=>$filename,
				'mimetype'=>mime_content_type($source.$filename),
				'bytes'=>filesize($source.$filename),
				'description'=>'Icebear screen definition file'
			);
			$baseObject=baseobject::create($request, 'file');
			$fileId=$baseObject['created']['id'];
			database::query('UPDATE file SET name=:name WHERE id=:id', array(':name'=>'file'.$fileId, ':id'=>$fileId));
			copy($source.$filename, $destinationDir.'file'.$baseObject['created']['id']);
			if(!empty($backupDir)){
				copy($source.$filename, $backupDir.'file'.$baseObject['created']['id']);
			}
			
			$screenExtensions=array('.csv','.txt','.doc','.docx','.xls','.xlsx','.pdf');
			foreach($screenExtensions as $ext){
				$filePath=$source.$screenName.$ext;
				if(file_exists($filePath)){
					$request['filename']=$screenName.$ext;
					$request['mimetype']=mime_content_type($filePath);
					$request['bytes']=filesize($filePath);
					$request['description']='Manufacturer screen definition file';
					$baseObject=baseobject::create($request, 'file');
					$fileId=$baseObject['created']['id'];
					database::query('UPDATE file SET name=:name WHERE id=:id', array(':name'=>'file'.$fileId, ':id'=>$fileId));
					copy($filePath, $destinationDir.'file'.$baseObject['created']['id']);
					if(!empty($backupDir)){
						copy($filePath, $destinationDir.'file'.$baseObject['created']['id']);
					}
				}
			}
		}
		$sourceDir->close();
		
		//parse the CSV for manufacturer, catalogue number
		$file=@fopen('refdata/screen/_screen.csv','r');
		if(!$file){ throw new Exception('Could not open screen list for import'); }
		$keys=fgetcsv($file);
		$values=fgetcsv($file);
		while(false!==$values){
			$screenName=$values[0];
			$screen=screen::getByName($screenName);
			$screenId=$screen['id'];
			$request=array();
			$request['isstandard']=1;
			for($i=1;$i<count($keys);$i++){ // 0 is name, not modifying
					$request[$keys[$i]]=$values[$i];
			}
			var_dump($request);
			screen::update($screenId, $request);
			$values=fgetcsv($file);
		}
		@fclose($file);
		
		database::commit();
	} catch (Exception $e){
		database::abort();
		throw new Exception($e->getMessage());
	}
	
	
}

?>